# Automatically adapted for numpy.oldnumeric Nov 22, 2012 by 

"""
NAM: ftest.py
DES: test python wrapped f90 methods
HIS: FB040502
     FB040509 adjusted to new f90 methods and call strategy. added timestamps
USE: execfile('fortran/ftest.py')

"""

# from numarray import array
from numpy.oldnumeric import array
from .fortran import f90
import time

# converting all back to Numeric if necessary

Flag   = (BoaB.currData.Data_Flag_p)
Data   = (BoaB.currData.Data_Red_p)
Mean   = (BoaB.currData.Cha_Mean)
Med    = (BoaB.currData.Cha_Med)
Rms    = (BoaB.currData.Cha_Rms)
Mean_s = (BoaB.currData.Cha_Mean_s)
Med_s  = (BoaB.currData.Cha_Med_s)
Rms_s  = (BoaB.currData.Cha_Rms_s)

Nph  = 2
Nch  = 0

Data_Bac_p = Data
Flag_Bac_p = Flag

plotc([Nch+1])
plotp(Nph)
BoaB.Graphic.PlotData.setPlotLimits(['*', '*', -2000, 10000])
cl()
set(ciData, 1)
set(lwBox, 5)
show(ciData)
signal()

#-- DEMONSTRATE USE OF get_subscan_index:
BoaB.currData.Subscan_Index = f90.baseline.get_subscan_index(BoaB.currData.Subscan_Index)
f90.data.subscan_index = BoaB.currData.Subscan_Index
print "Subscan_Index=", BoaB.currData.Subscan_Index
print f90.data.subscan_index 

#-- DEMONSTRATE USE OF flag_pc:  (_pc means phase+channel)
print f90.chanana.flag_pc.__doc__
Flag = f90.chanana.flag_pc(Flag, Data, Nph, Nch, 2500, 3500)
print "new flag:", Flag[:, Nph, Nch]

#-- DEMONSTRATE USE OF flag:
print f90.chanana.flag.__doc__
Flag[:, Nph, Nch], count = f90.chanana.flag(Flag[:, Nph, Nch], Data[:, Nph, Nch], 2600, 3400)
print count, " new flag:", Flag[:, Nph, Nch]

BoaB.currData.Data_Flag_p = Flag

#-- DEMONSTRATE USE OF compress:
compress_array, nmax = f90.f1.compress(Data[:, Nph, Nch], Flag[:, Nph, Nch], 0)
print "nmax", nmax
print compress_array[0:nmax]

#-- DEMONSTRATE USE OF compress:
# input_array = Numeric.array(range(5),'f')
# flag_array  = Numeric.array([0,1,0,1,0],'i')
# compress_array,nmax = f90.f1.compress(input_array,flag_array,1)
# compress_array = compress_array[0:nmax]
# print "nmax=",nmax," compressed array=",compress_array

#-- DEMONSTRATE USE OF mean_rms:
t0 = time.clock()
Mean, Med, Rms = f90.chanana.mean_rms(Data, Flag, Mean, Med, Rms)
t1 = time.clock()
print "TIME(mean_rms)  "+str(t1-t0)
print "mean=", Mean[:, Nch]
print "median=", Med[:, Nch]
print "rms=", Rms[:, Nch]
print "mean=", Mean[Nph,:]

BoaB.currData.Cha_Mean = Mean  
BoaB.currData.Cha_Med = Med
BoaB.currData.Cha_Rms = Rms

#-- DEMONSTRATE USE OF addpoly:
# Poly = [20.,-20.,5.,10.,-5.,5.,-5.]
Poly = [10., 0., 10., 20.]
DataAdd = Data
for iph in range(3):
    DataAdd = f90.baseline.addpoly(DataAdd, Poly, Mean, Rms, iph, Nch) 

BoaB.currData.Data_Red_p = DataAdd
set(ciData, 1)
set(symbolPoint, 3)
set(chPoint, 20)
signal()

#-- DEMONSTRATE USE OF mean_rms_s:
t0 = time.clock()
DataMean = array(DataAdd)
DataMean, Mean_s, Med_s, Rms_s = f90.chanana.mean_rms_s(DataMean, Flag, Mean_s, Med_s, Rms_s)
t1 = time.clock()
print "TIME(mean_rms_s)  "+str(t1-t0)

print "mean=", Mean_s[Nph, Nch,:]
print "median=", Med_s[Nph, Nch,:]
print " rms=", Rms_s[Nph, Nch,:]
BoaB.currData.Data_Red_p = DataMean
set(ciData, 6)
signal()

Poly = array(range(2), 'f')
DataFit = array(DataAdd)
DataFit, Poly = f90.baseline.polyfit(DataFit, Poly, Flag, Mean, Rms, Nph, Nch) 
BoaB.currData.Data_Red_p = DataFit
set(ciData, 2)
set(symbolPoint, 1)
signal()

Poly = array(range(3), 'f')
DataFit = array(DataAdd)
DataFit, Poly = f90.baseline.polyfit(DataFit, Poly, Flag, Mean, Rms, Nph, Nch) 
BoaB.currData.Data_Red_p = DataFit
set(ciData, 3)
signal()

Poly = array(range(4), 'f')
DataFit = array(DataAdd)
DataFit, Poly = f90.baseline.polyfit(DataFit, Poly, Flag, Mean, Rms, Nph, Nch) 
BoaB.currData.Data_Red_p = DataFit
set(ciData, 4)
signal()

BoaB.currData.Data_Red_p = DataAdd-DataFit
set(ciData, 5)
signal()

# print Fit[:,Nph,Nch]
# print Poly

# print f90.data.data_red_p[:,Nph,Nch]
# BoaB.currData.Data_Red_p = f90.data.data_red_p
# print BoaB.currData.Data_Red_p[:,Nph,Nch]

# f90.test2.foo(1,2)
# f90.data.data_red_p[0:2,0,1] = 1000.
# print f90.data.data_red_p[:,1,0]
# print f90.data.data_red_p[0:2,1,0]
# print f90.test1.__doc__ 
# f90.test1.trythis()

# restore initial data:
BoaB.currData.Data_Red_p  = Data_Bac_p
BoaB.currData.Data_Flag_p = Flag_Bac_p

# BoaB.currData.Data_Red_p = DataAdd


#-------------- test reimage:

from Numeric import *
from .fortran import f90
im = zeros((11, 11), 'f')
data = array([10, 20, 30], 'f')
xy  = array([ [-50, 0, 20], [-30, 0, 20] ], 'f')
dxy = array([ [11, 6, 11], [11, 6, 11] ], 'f')
zerooffset = array([-50, -50], 'f')
xyscale =  array([0.1, 0.1], 'f')
im = f90.f1.reimage(im, data, xy, dxy, zerooffset, xyscale)
print im









# macros.py
#
# BoA macros to reduce maps, pointing, focus, clean data
#
# 2007-03-24 FB fixed clean and map. changed names to macMethod

import os

# datadir = os.environ.get("BOA_HOME_EXAMPLES")
# datadir = os.environ.get("APEXRAWDATA")
# if datadir:
#    indir(datadir)
# proj('T-77.F-0002-2006')
# proj('T-79.F-0010-2007')


#

def macF(ScanNum='4451', refChan=102):
    """
    Quick focus reduction script
    first solves focus without, then with simple skynoise subtraction
    reference channel for SNF should be different from the pointing channel

    USAGE:  macF('4451',refChan=102)
    """

    read(ScanNum, febe='BOLOSZ-SZACBE')
    data.ReceiverArray.RefChannel = refChan
    medianbase(subscan=0)
    signal(range(refChan - 3, refChan + 2))
    print "..SOLVE FOCUS .. PRESS <Enter>"
    raw_input()
    solveFoc()
    # when 102 is reference then noise filter reference=100 !!!
    medianNoiseRemoval(chanRef=100)
    print "..NOW SNR .. PRESS <Enter>"
    raw_input()
    signal(range(refChan - 3, refChan + 2))
    print "..SOLVE FOCUS .. PRESS <Enter>"
    raw_input()
    solveFoc()

#


def macP(ScanNum='4451', refChan=102, flagChannels=[], radius=0,
         Xpos=0, Ypos=0, newrcp=1, solve=1, plot=1, png=0, oversamp=2):
    """
    Quick pointing reduction script.
    When pointing on a non-ref channel (ie not the one that the rcp's are made for)
    then, then make map with the number of the pointing channel
    Note that solvePointing and solvePointingOnMap yield slightly different results.
    We are not sure which on is right.
    OnMap is more robust, thus take that for now.

    USAGE: macP('4451',refChan=102)
    """

    read(ScanNum, febe='BOLOSZ-SZACBE')
    flagCh([143])
    if newrcp:
        updateRCP('aszca-4522.rcp')
    # we assume that the MBFITS gives the right refChan=102
    # data.ReceiverArray.RefChannel = refChan
    medianBase(subscan=1)

    if np.size(data.ScanParam.SubscanNum) > 1:
        data.flagInTime('lst', below=0)

    signal([refChan])
    print "... NOW showing channel map..PRESS <Enter>"
    raw_input()
    mapping([refChan], oversamp=oversamp, noPlot=0, style='idl2')

    if solve:
        print "... NOW Pointing on map"
        data.solvePointingOnMap([refChan], plot=1, style='idl2')
        print "... NEXT Pointing on data ....PRESS <Enter>"
        raw_input()
        data.solvePointing(
            [refChan],
            plot=1,
            radius=radius,
            Xpos=Xpos,
            Ypos=Ypos,
            fixedPos=0)


#

def macLoop(ScanList=[], plot=0, png=1, rmsClip=2):

    for scan in ScanList:
        try:
            macClean(
                ScanNum=str(scan),
                plot=plot,
                png=png,
                rmsClip=rmsClip,
                gain=0)
        except:
            print "********************************"
            print "*"
            print "* Reduction of scan %s failed!" % str(scan)
            print "*"
            print "********************************"

#


def macClean(ScanNum='4534', newrcp=1, despike=1, flagChannels=[], rmsClip=0,
             plot=1, png=0, gain=0, baseOrder=0, rcpfile='aszca-4639-NEW.rcp'):
    """
    Script to read & clean data, ie. view and remove bad channels, baseline.

    USAGE: clean('4534',flagChannels=[34, 82, 87])
    """

    from .fortran import fStat

    pngFiles = []  # The Names of the png files to be created
    pngDir = '/data2/boa2/aszca/'  # The directory where the png files will be stored

    # indir('./Examples')
    # proj('T-77.F-0002-2006')

    if(ScanNum):
        read(ScanNum, febe='BOLOSZ-SZACBE')

    BogliConfig.point['size'] = 0.01

    # flagC([113,167,169,176])  # flag open channels for LABOCA

    if newrcp:
        print "reading in new rcp file", rcpfile
        flagChannelsRcp = updateRCP(rcpfile)
        flagChannels = concatenate((flagChannels, flagChannelsRcp))
        print 'bad channels according to rcp file:'
        print flagChannels

    if np.size(data.ScanParam.SubscanNum) > 1:
        data.flagInTime('lst', below=0)  # sometimes there are large negatives

    print "reference channel=  " + repr(data.ReceiverArray.RefChannel)

    medianBaseline(subscan=0)  # 0 ORDER BASELINE
    medRms = 8. * fStat.f_median(data.getChanListData(type='rms'))

    chans = array([])
    if plot:
        print "... NEXT PLOTTING BAD CHANNELS ...................."
        signal(chanList=flagChannels, limitsY=[-medRms, medRms])
        new = '1'
        while new != '':
            print "... GIVE CHANNEL NUMBER NOT TO FLAG OR -1 TO UNFLAG ALL OR <ENTER>"
            new = raw_input()
            if new != '':
                chans = concatenate((chans, [int(new)]))
                print chans
                if int(new) == -1:
                    flagChannels = []

    if png:
        name = ScanNum + "-signal-0.ps"
        print name
        op(name + '/CPS')
        signal(
            chanList=flagChannels,
            limitsY=[-medRms,
                     medRms],
            overplot=0,
            ci=1,
            style='l')
        close()
        print "Execute ps2png90 800 " + ScanNum + "-signal-0 &"
        os.system("ps2png90 800 " + ScanNum + "-signal-0 &")
        pngFiles.append(ScanNum + "-signal-0.png")

    if flagChannels:
        flagC(chanList=flagChannels)    # FLAG BAD CHANNELS
        unflagC(chanList=chans)         # UNFLAG SOME AGAIN

    if gain:
        print "... APPLYING GAIN CORRECTION TO UNFLAGGED CHANNELS"
        for i in range(320):
            if not data.ReceiverArray.FlagHandler.isSetOnIndex(i):
                data.Data[:, i] = array(
                    data.Data[:, i] / data.ReceiverArray.Gain[i]).astype(Float32)

    base(order=baseOrder, subscan=0)
    # medianBaseline(subscan=0)        # 0 ORDER BASELINE ALL CHANNELS
    stat()              # compute statistics
    if despike:
        data.despike()        # despike

    if plot:
        print "... NEXT PLOTTING GOOD CHANNELS ........."  # ; raw_input()
        signal(overplot=0, ci=1, style='p')

    if png:
        name = ScanNum + "-signal-1.ps"
        print name
        op(name + '/CPS')
        signal(overplot=0, ci=1, style='l')
        close()
        print "Execute ps2png90 800 " + ScanNum + "-signal-1 &"
        os.system("ps2png90 800 " + ScanNum + "-signal-1 &")
        pngFiles.append(ScanNum + "-signal-1.png")

    rmsbelow = 0
    rmsabove = 0
    medianRms = fStat.f_median(data.getChanListData(type='rms'))

    if plot:                                      # SHOW RMS OF CHANNELS
        print "... NEXT PLOTTING RMS OF CHANNELS .. PRESS <Enter>"
        raw_input()
        BogliConfig.point['size'] = 5
        data.plotRmsChan()
        BogliConfig.point['size'] = 0.01

        line = medianRms * array([1., 1.])
        Plot.plot([0, 330], line, overplot=1, ci=2, style='l', ls=1, width=2)
        if rmsClip:
            line = medianRms / rmsClip * array([1., 1.])
            Plot.plot([0, 330], line, overplot=1, ci=2, style='l', ls=4)
            line = medianRms * rmsClip * array([1., 1.])
            Plot.plot([0, 330], line, overplot=1, ci=2, style='l', ls=4)

        print "... MEDIAN=" + repr(fStat.f_median(data.getChanListData(type='rms')))

        if not rmsClip:
            print "... GIVE SELECTION THRESHOLD ABOVE:"
            rmsabove = raw_input()
            print "... GIVE SELECTION THRESHOLD BELOW:"
            rmsbelow = raw_input()

    if rmsClip:
        if not rmsbelow:
            rmsbelow = medianRms / rmsClip
        if not rmsabove:
            rmsabove = rmsClip * medianRms

    if rmsbelow:
        data.flagRms(below=float(rmsbelow))   # FLAG DEAD CHANNELS BELOW VALUE
    if rmsabove:
        data.flagRms(above=float(rmsabove))           # FLAG NOISY CHANNELS

    if plot:
        # print "... NEXT PLOTTING SIGNAL .... PRESS <Enter>"; raw_input()
        signal(overplot=0, ci=1, style='p')
        chans = array([])
        new = '1'
        while new != '':
            print "... ADD A CHANNEL TO FLAG ..................... OR <ENTER>"
            new = raw_input()
            if new != '':
                chans = concatenate((chans, [int(new)]))
                print chans
        if chans:
            chans = array(chans)
            data.flagChannels(chans)
            # signal(overplot=0,ci=1,style='p')

    if png:
        name = ScanNum + "-signal-2.ps"
        op(name + '/CPS')
        signal(overplot=0, ci=1, style='l')
        close()
        print "Execute ps2png90 800 " + ScanNum + "-signal-2 &"
        os.system("ps2png90 800 " + ScanNum + "-signal-2 &")
        name = ScanNum + "-signal-3.ps"
        op(name + '/CPS')
        signal([100, 102, 103], style='l')
        close()
        print "Execute ps2png90 500 " + ScanNum + "-signal-3 &"
        os.system("ps2png90 500 " + ScanNum + "-signal-3 &")
        pngFiles.append(ScanNum + "-signal-2.png")
        pngFiles.append(ScanNum + "-signal-3.png")

    if plot:
        print "... NEXT PLOTTING CORRELATION .... GIVE REF CHANNEL or -1: "
        chan = raw_input()
        if chan:
            plotCorrel(chanRef=int(chan))

        print "... NEXT PLOTTING CORRELATION MATRIX .. PRESS <Enter>"
        raw_input()
        data.plotCorMatrix(distance=1)

    if png:
        # Create the html file in pngDir:
        htmlFilename = os.path.join(pngDir, ScanNum + ".html")

        # open is overwritten in boa!
        import __builtin__

        print "Writing", htmlFilename
        htmlFile = __builtin__.open(htmlFilename, "w")
        htmlFile.write("<html>\n")
        htmlFile.write("<body>\n")
        for pngFile in pngFiles:
            htmlFile.write("<img src=%s><br>\n" % pngFile)
        htmlFile.write("</body>\n")
        htmlFile.write("</html>\n")
        htmlFile.close()

#


def macAzel(ScanNum='4534', plot=1, png=0):
    """
    USAGE: macAzel('4534')
    """

    from .fortran import fStat

    azimuth = data.ScanParam.get('AzOff')
    elevation = data.ScanParam.get('ElOff')
    azStat = fStat.f_stat(azimuth)
    limitsX = array([-1, 1]) * 2.5 * azStat[1] + azStat[0]
    elStat = fStat.f_stat(elevation)
    limitsY = array([-1, 1]) * 2.5 * elStat[1] + elStat[0]

    if plot:
        azeloff(style='p', limitsX=limitsX, limitsY=limitsY, aspect=0)

    if png:
        name = ScanNum + "-azeloff.ps"
        op(name + '/ps')
        BogliConfig.point['size'] = 2
        azeloff(style='p', limitsX=limitsX, limitsY=limitsY, aspect=0)
        BogliConfig.point['size'] = 0.01
        close()
        os.system("ps2png90 600 " + ScanNum + "-azeloff &")

 #


def macMap(ScanNum='4534', solve=0, despike=0, flagChannels=[], rmsClip=0,
           plot=1, png=0, radius=0, gain=1, weight=1):
    """
    script for simple map reduction

    USAGE: macMap('4534',flagChannels=[ 96,133 ],plot=0,rmsClip=2)
           style : g2r r2g r2b blue gray grey inverse idl2 idl4 idl5
                    (see BogliConfig.lutDir)
    """

    from .fortran import fStat

    macClean(
        ScanNum=ScanNum, newrcp=1, despike=despike, flagChannels=flagChannels,
             rmsClip=rmsClip, plot=plot, png=png)

    print "... NEXT SHOWING AZELOFF ....... PRESS <Enter>"
    raw_input()
    macAzel(ScanNum=ScanNum, plot=plot, png=png)

    if weight:
        computeWeight()

    if plot:
        print "... NEXT SHOWING FIRST LOOK MAP ..........  press <Enter>"
        raw_input()
        mapping()            # make a first-look map
        print "... IF YOU WANT TO FLAG A SOURCE, GIVE RADIUS ..... OR <ENTER>:"
        radius = raw_input()
        if radius != '':
            flagPos(radius=float(radius), Az=0, El=0)
                    # Flag source position. Default flag=5

    print "... NOW BASELINE 1 AND NOISE REMOVAL ...."
    base(order=1)
    medianNoiseRemoval()
    # skynoise(ScanNum=ScanNum,ChanRef=ChanRef,plot=plot, radius=float(radius))

    if plot:
        signal(overplot=0, ci=1, style='l')

    if png:
        name = ScanNum + "-signal-sn.ps"
        op(name + '/ps')
        signal(overplot=0, ci=1, style='l')
        close()
        os.system("ps2png90 700 " + ScanNum + "-signal-sn &")

    if plot:
        print "... NEXT SHOWING RMS................... PRESS <Enter>"
        raw_input()
        BogliConfig.point['size'] = 5
        plotRmsChan()
        BogliConfig.point['size'] = 0.01

    if png:
        BogliConfig.point['size'] = 5
        name = ScanNum + "-rms-sn.ps"
        op(name + '/ps')
        data.plotRmsChan()
        close()
        os.system("ps2png90 600 " + ScanNum + "-rms-sn &")
        BogliConfig.point['size'] = 0.01

    print "... MEDIAN=" + repr(fStat.f_median(data.getChanListData(type='rms')))

    first3 = data.ReceiverArray.checkChanList(
        [])[0:3]    # select 3 channels for FFT
    if plot:
        print "... NEXT SHOWING FFT ............ PRESS <Enter>"
        raw_input()
        data.plotFFT(first3)

    if png:
        name = ScanNum + "-fft-sn.ps"
        op(name + '/ps')
        data.plotFFT(first3)
        close()
        os.system("ps2png90 600 " + ScanNum + "-fft-sn &")

    if radius:
        data.unflag(flag=5)   # unflag the previously flagged circle

    if plot:
        print "... NEXT SHOWING MAP.................. PRESS <Enter>"
        raw_input()
    #   data.horizontalMap()
        mapping()

    if png:
        name = ScanNum + "-map-sn.ps"
        op(name + '/cps')
        mapping()
        # Map.display(style='idl4')
        close()
        os.system("ps2png90 600 " + ScanNum + "-map-sn &")

    ChanRef = data.ReceiverArray.RefChannel

    if plot:
        print "... NEXT SHOWING SIGNAL PLOT REF CHAN.........PRESS <ENTER>"
        raw_input()
        #    data.Data = data.DataBackup
        signal(chanList=[ChanRef], ci=1, style='p')
        #    data.Data = Data_SNR
        #    signal(chanList=[ChanRef],overplot=1,ci=2,style='l')

    if png:
        name = ScanNum + "-signal-ref.ps"
        op(name + '/ps')
        signal(chanList=[ChanRef], ci=1, style='p')
        close()
        os.system("ps2png90 400 " + ScanNum + "-signal-ref &")


#

def macPoint(
    ScanNum='42947',
     ChanRef=102,
     rmsClip=0,
     flagChannels=[],
     radius=0,
     Xpos=0,
     Ypos=0,
     newrcp=1,
     solve=0,
     despike=0,
     plot=1,
     png=0):
    """
    USAGE: execfile('macros.py')
           macPoint('42947')          # strong source
           macPoint('46117',Rad=200)  # fainter source
    """

    # 228 p Jup high noise, off pointed, good for test of SN
    # 229 p Jup noisy, offpointed
    # 252 p Jup low noise

    badChan = [
        1,
        3,
     4,
     5,
     6,
     7,
     8,
     12,
     13,
     14,
     15,
     16,
     56,
     62,
     65,
     70,
     77,
     140,
     79,
     80,
     237,
     288]  # 42947 137

    macClean(
        ScanNum=ScanNum,
        newrcp=1,
     despike=despike,
     flagChannels=badChan,
     rmsClip=rmsClip,
     plot=plot,
     png=png)

    macAzel(ScanNum=ScanNum, plot=plot, png=png)

    if plot:
        # data.equatorialMap(aspect=1)
        print "... NEXT SHOWING MAP................................... PRESS <Enter>"
        raw_input()
        mapping(oversamp=3)
        if not radius:  # IF RADIUS IS NOT ALREADY SET
            print "... TO FLAG A SOURCE FOR base AND Noise, GIVE RADIUS ............ OR <ENTER>:"
            radius = raw_input()

    if radius:
        flagPos(radius=float(radius))  # FLAG SOURCE POSITION. DEFAULT flag=5

    print "... NOW BASELINE 1 AND NOISE REMOVAL ...."
    base(order=1)
    medianNoiseRemoval()
    # skynoise(ScanNum=ScanNum,ChanRef=ChanRef,plot=plot, radius=float(radius))

    if radius:
        data.unflag(flag=5)   # UNFLAG THE PREVIOUSLY FLAGGED CIRCLE

    if solve:
        solvepointing(plot=1, radius=-10, Xpos=Xpos, Ypos=Ypos, circular=0)
        # radius<0 means multiple of beam FWHM
        if png:
            BogliConfig.box['color'] = 4
            BogliConfig.box['linewidth'] = 3
            name = ScanNum + "-map-sn.ps"
            op(name + '/cps')
            data.showPointing(style='idl2')
            close()
            BogliConfig.box['color'] = 1
            BogliConfig.box['linewidth'] = 1
            os.system("ps2png90 600 " + ScanNum + "-map-sn &")

    if plot:
        print "... NEXT SHOWING REF CHANNEL MAP .......................... PRESS <Enter>"
        raw_input()
        # data.equatorialMap([ChanRef],aspect=1)
        mapping([ChanRef], system='HO')

    if solve:
        # data.solvePointing(radius=radius,plot=plot,chanList=[ChanRef],Xpos=Xpos,Ypos=Ypos)
        data.solvePointing(
            plot=1,
            radius=-10,
            Xpos=Xpos,
            Ypos=Ypos,
            circular=0,
            chanList=[ChanRef])
        if png:
            BogliConfig.box['color'] = 3
            BogliConfig.box['linewidth'] = 2
            name = ScanNum + "-map-sn-ref.ps"
            op(name + '/cps')
            data.showPointing(style='idl2')
            close()
            BogliConfig.box['color'] = 1
            BogliConfig.box['linewidth'] = 1
            os.system("ps2png90 600 " + ScanNum + "-map-sn-ref &")

    # skynoise(ChanRef=ChanRef,plot=plot)

#    if plot:
#        data.equatorialMap(aspect=1)
#    if solve:
#        data.solvePointing (radius=Rad,plot=plot,chanList=[],Xpos=Xpos,Ypos=Ypos)
# print "...  NEXT SHOWING SIGNAL PLOT  ......................PRESS
# <Enter>" ; raw_input()

#    Data_SNR = data.Data
#    data.Data = data.DataBackup
#    signal(ci=1,style='p')
#    data.Data = Data_SNR
#    signal(overplot=1,ci=2,style='p')

    if plot:
        print "... NEXT SHOWING SIGNAL PLOT REF CHAN.................PRESS <ENTER>"
        raw_input()
        #    data.Data = data.DataBackup
        signal(chanList=[ChanRef], ci=1, style='l')
        #    data.Data = Data_SNR
        #    signal(chanList=[ChanRef],overplot=1,ci=2,style='l')

    if png:
        name = ScanNum + "-signal-ref.ps"
        op(name + '/ps')
        signal(chanList=[ChanRef], ci=1, style='l')
        close()
        os.system("ps2png90 400 " + ScanNum + "-signal-ref &")

#


def macSkydip(ScanNum='316', ChanRef=0, plot=1, solve=1):
    """
    Reduce a skydip observation
    """
    read(ScanNum)

    if not data.ReceiverArray.checkChanList(array(ChanRef)):
        ChanRef = data.ReceiverArray.RefChannel

    # Retrieve signal and elevation
    s = data.getChanData('flux', ChanRef)
    e = data.getChanData('el')
    e = (90. - e) * pi / 180.  # convert to zenithal distance, in radians
    if plot:
        Plot.plot(
            1. / cos(e),
            s,
            labelX='Airmass',
            labelY='Counts',
            caption=data.ScanParam.caption())

    if solve:
        # TODO: linear fit
        pass

#


def macCmpRms(scan, pos=0, window=100, outdir='', pngplot=0):
    """
    Estimate RMS for a given observation using median filter
    """

    read(str(scan), febe='BOLOSZ-SZACBE')

    medianBaseline()

    doPlot = ((outdir != '') & pngplot)
    if doPlot:
        op(outdir + '/' + str(scan) + '_medianbaseline.ps/CPS')
    r = data.medianFilter(plot=1, limitsY=[-60, 60], subtract=1, window=window)
    if doPlot:
        close()
        os.system("ps2png90 600 " + outdir +
                  '/' + str(scan) + '_medianbaseline &')

    if pos != 0:
        updateRCP('aszca-' + str(scan) + '-uncalib.rcp')
        data.flagPosition(pos, radius=180)

    itDespike(below=-4, above=4)

    c = checkChanList([])
    rms = data.getChanListData('rms')

    if (outdir != ''):
        a = file(outdir + '/Channel-RMS_' + str(scan) + '.dat', 'w')
        for i in range(len(c)):
            a.write(str(c[i]) + ' ')
            a.write(str(rms[i]))
            a.write('\n')
        a.close()

    if doPlot:
        op(outdir + '/' + str(scan) + '_rms.ps/CPS')
    signal(limitsY=[-60, 60])
    if pngplot:
        close()
        os.system("ps2png90 600 " + outdir + '/' + str(scan) + '_rms &')


#



# Copyright (C) 2014
# Institut d'Astrophysique Spatiale
#
# Forked from the BoA project
#
# Copyright (C) 2002-2006
# Max-Planck-Institut fuer Radioastronomie Bonn
# Argelander Institut fuer Astronomie
# Astronomisches Institut der Ruhr-Universitaet Bochum
#
# Produced for the LABOCA project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#

"""
..modules:: ReaCommandHistory.py (module)
:synopsis: provides methods to access the command history
"""
__version__ = '$Revision: 1730 $'
__date__ = '$Date: 2006-09-26 22:16:08 +0200 (Tue, 26 Sep 2006) $'

#----------------------------------------------------------------------------------
#----- Import ---------------------------------------------------------------------
#-------------------------------------------------------------------------
import readline

from rea import ReaMessageHandler

#----------------------------------------------------------------------------------
#----- _CommandHistory Class ------------------------------------------------------
#-------------------------------------------------------------------------


class _CommandHistory:

    """..class: CommandHistory
    :synopsis: Encapsulate calls to the readline module to access tho command history
    """

    def __init__(self):
        self._messHand = ReaMessageHandler.MessHand(self.__module__)

        self._hasGetHistoryItem = 0

        try:
            readline.read_history_file()
        except IOError:
            self._messHand.warning(
                "Not able to retrieve last session commands")
            return

        if not hasattr(readline, "get_history_item"):
            # For Python earlier than Version 2.3
            self._messHand.warning(
                "Command history not avaiable for this Python version")
            return
        else:
            self._hasGetHistoryItem = 1

            self._tags = []

            try:
                curHistLen = readline.get_current_history_length()
            except:
                curHistLen = 0

            if curHistLen:
                self._tags.append(curHistLen + 1)
            else:
                self._tags.append(1)

    def __del__(self):
        try:
            readline.write_history_file()
        except IOError:
            self._messHand.warning("Not able to save session commands")

    def addTag(self):
        if self._hasGetHistoryItem:
            try:
                self._tags.append(readline.get_current_history_length())
            except:
                self._messHand.warning(
                    "Not able to add tag to command history")

    def getCommands(self, itag=0):
        ret = []
        if self._hasGetHistoryItem:
            try:
                iStop = readline.get_current_history_length()
                for i in xrange(self._tags[itag], iStop + 1):
                    ret.append(readline.get_history_item(i))
            except:
                self._messHand.warning("Not able to get command history")
        return ret


#----------------------------------------------------------------------------------
#----- The only _CommandHistory object --------------------------------------------
#-------------------------------------------------------------------------

_theHistory = None

#----------------------------------------------------------------------------------
#----- The interface of the module ------------------------------------------------
#-------------------------------------------------------------------------


def initHistory():
    """Initialize the command history from the .history file"""
    global _theHistory
    if not _theHistory:
        _theHistory = _CommandHistory()


def tagHistory():
    """Adds a tag to the command history. Tags are integer numbers starting at 0. """
    global _theHistory
    if _theHistory:
        _theHistory.addTag()


def getHistory(itag=0):
    """returns the command history starting at tag itag

    Parameters
    ----------
    itag : int
        Tag from whereon commands are returned.  For itag=0, all
        command since the call of initHistory are returned; for
        itag=-1, the command since the last call of tagHistory are
        returned.

    Returns
    -------
    list of str
        Commands in the history
    """
    global _theHistory
    if _theHistory:
        return _theHistory.getCommands(itag)
    else:
        return None


def writeHistoryToFile(filename, itag=0):
    """Write command history to the specified file

    Parameters
    ----------
    filename : str
        The full filename
    itag : int
        See ReaCommandHistory.getHistory
    """
    global _theHistory
    if _theHistory:
        of = open(filename, 'w')
        commands = getHistory(itag)
        for command in commands:
            of.write("%s\n" % command)
        of.close()
    else:
        self._messHand.warning("Not able to write history to file")



# Copyright (C) 2014
# Institut d'Astrophysique Spatiale
#
# Forked from the BoA project
#
"""
NAM: ReaMBFitsReader.py (file)
DES: Provides classes and methodes for the high level access to MBFits datasets.
     The Module Interface consists of
     - Function createReader
     - Class MBFitsReader
     - Class ApexMBFitsReader
     - Class IramMBFitsReader
     - Class MBFitsReaderError

     MBFitsReader is the parent class of ApexMBFitsReader and IramMBFitsReader and
     contains the public interface for the subclasses. To read the contents of a MBFits
     dataset, use the concrete subclasses of MBFitsReader.
"""
# ---------------------------------------------------------------------
# ---- Import ---------------------------------------------------------
# ---------------------------------------------------------------------

from . import ReaMBFits
import numpy as np

# ---------------------------------------------------------------------
# ---- Function createReader ------------------------------------------
# ---------------------------------------------------------------------

def createReader(dataset):
    """
    DES: Creates a new MBFitsReader for the specified ReaMBFits.Dataset object dataset.
         Depending on the contents of the dataset, either an ApexMBFitsReader or an
         IramMBFitsReader will be created.
    INP: (ReaMBFits.Dataset) dataset: The dataset to be read by the created MBFitsReader.
    OUT: (MBFitsReader)             : The reader object of the proper concrete subtype of
                                      MBFitsReader.
    """
    try:
        kwMBFTSVER = dataset.getKeyword("MBFTSVER")
        if kwMBFTSVER:
            return ApexMBFitsReader(dataset)

        kwIMBFTSVE = dataset.getKeyword("IMBFTSVE")
        if kwIMBFTSVE:
            return IramMBFitsReader(dataset)
    except Exception as data:
        raise MBFitsReaderError(str(data))


# ---------------------------------------------------------------------
# ---- Class MBFitsReader ---------------------------------------------
# ---------------------------------------------------------------------

class MBFitsReader:
    """
    DES: Parent reader class for ApexMBFitsReader and IramMBFitsReader.
         Contains the public interface for the subclasses and common private methods.
         To read the contents of a MBFits dataset, use the concrete subclasses of this
         class.
    """

    def __init__(self, dataset):
        self._dataset = dataset

        # The dictionary that maps item keys to functions and arguments are
        # filled in the constructors of the subclasses
        self._itemKeyToFuncArgsMap = {}

    # -----------------------------------------------------------------

    def getType(self):
        return self.__class__.__name__

    # -----------------------------------------------------------------

    def getBlankInt(self):
        """
        DES: Returns blanking value for integers as used in MBFITS
        """
        return -999

    # -----------------------------------------------------------------

    def getBlankFloat(self):
        """
        DES: Returns blanking value for floats as used in MBFITS
        """
        return -999.

    # -----------------------------------------------------------------

    def openSubscan(self, subsnum=None):
        """
        DES: Opens all tables related with the specified subscan.
        INP: (int) subsnum: The number of the subscan to be opened.
                            If subsnum = None (the default) all tables
                            that are related with the scan itself instead
                            of a subscan are opened.
        OUT: (int)        : The number of tables opened
        """
        try:
            if subsnum == None:
                tables = self._dataset.getTables(OBSNUM=None)
            else:
                tables = self._dataset.getTables(OBSNUM=subsnum)

            nOpenedTables = 0
            for table in tables:
                if not table.isOpen():
                    nOpenedTables += 1
                    table.open()
            return nOpenedTables
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def closeSubscan(self, subsnum=None):
        """
        DES: Closes all (open) tables related with the specified subscan.
        INP: (int) subsnum: The number of the subscan to be closed.
                            If subsnum = None (the default) all tables
                            that are related with the scan itself instead
                            of a subscan are closed.
        """
        try:
            if subsnum == None:
                tables = self._dataset.getTables(OBSNUM=None)
            else:
                tables = self._dataset.getTables(OBSNUM=subsnum)
            for table in tables:
                if table.isOpen():
                    table.close()
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def read(self, itemKey, **kargs):
        """
        DES: Reads item itemKey from the dataset, using the additional
             arguments in **kargs.
        INP: (string) itemKey: The key of the item to be read.
                               Inspect the init method of the concrete
                               subclasses for valid keys.
                      **kargs: Additional arguments necessary to read
                               the specified item.
                               Inspect the init method of the concrete
                               subclasses for necessary additional arguments.
        OUT:                 : The read item.
                               May be of virtually every data type depending
                               on the datatype and location of the item in the
                               MBFitsFile.
        """
        try:
            # Get the information from _itemKeyToFuncArgsMap:
            fa =  self._itemKeyToFuncArgsMap[itemKey]
            func = fa["function"]
            setArgs = fa["setArgs"]
            addArgs = fa["addArgs"]

            # Check that all necessary argumets are passed:
            for addArg in addArgs:
                if not addArg in kargs.keys():
                    raise "Argument '%s' missing" % addArg
                setArgs[addArg] = kargs[addArg]

            # Retrieve the data using the function specified in
            # _itemKeyToFuncArgsMap:
            result = func(*[], **setArgs)
            return result

        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readKeyword(self, key, extname=None, **kargs):
        # Read a keyword:
        try:
            if not extname:
                kw = self._dataset.getKeyword(key)
            else:
                tbl = self._getTable(extname=extname, **kargs)
                kw = tbl.getKeyword(key)
            if not kw:
                raise "Could not read keyword"
            result = kw.getValue()
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readKeywordTuple(self, keys, extname=None, **kargs):
        # Read a tuple of keywords:
        try:
            result = []
            for key in keys:
                result.append(self._readKeyword(key=key, \
                                                extname=extname, \
                                                **kargs))
            return tuple(result)
        except Exception as data:
            raise MBFitsReaderError(str(data))

      # -----------------------------------------------------------------

    def _readKeywordFebe(self, key, extname=None, **kargs):
        # Read a keyword and return a list as readColumnFebepar... :
        try:
            if not extname:
                kw = self._dataset.getKeyword(key)
            else:
                tbl = self._getTable(extname=extname, **kargs)
                kw = tbl.getKeyword(key)
            if not kw:
                raise "Could not read keyword"
            result = kw.getValue()
            return [result]
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readColumn(self, colname, extname, **kargs):
        # Read a column:
        try:
            tbl = self._getTable(extname=extname, **kargs)
            col = tbl.getColumn(colname)
            if not col:
                raise "Could not read column"
            result = col.read()
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _getTable(self, **kargs):
        try:
            kargsT = {}
            if "extname" in kargs.keys():
                kargsT["EXTNAME"] = kargs["extname"]
            if "subsnum" in kargs.keys():
                kargsT["OBSNUM"] = kargs["subsnum"]
            if "febe" in kargs.keys():
                kargsT["FEBE"] = kargs["febe"]
            if "baseband" in kargs.keys():
                kargsT["BASEBAND"] = kargs["baseband"]

            tbls = self._dataset.getTables(**kargsT)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            return tbls[0]
        except Exception as data:
            raise MBFitsReaderError(str(data))

# ---------------------------------------------------------------------
# ---- Class MBFitsReaderError ----------------------------------------
# ---------------------------------------------------------------------

class MBFitsReaderError(BaseException):

    def __init__(self, msg):
        self.msg = msg

    # -----------------------------------------------------------------

    def __str__(self):
        return repr(self.msg)




# ---------------------------------------------------------------------
# ---- Class ApexMBFitsReader -----------------------------------------
# ---------------------------------------------------------------------

class ApexMBFitsReader(MBFitsReader):
    """
    DES: Reader class for (APEX) MBFITS 1.60 and earlier.
         Consult the documentation of the superclass MBFitsReader
         and the source code of the init method to find out what the
         class does.
    """

    def __init__(self, dataset):
        MBFitsReader.__init__(self, dataset)

        # The following dictionary maps each itemKey to
        # a method of MBFitsReader together with information
        # about arguments of the call to this method.

        # Each entry into the dictionary must be of the form:
        # itemKey: {"function": function,
        #           "setArgs": setArgsDict,
        #           "addArgs": addArgsList}

        # itemKey:  The key to be used in the read method
        # function: The method of MBFitsReader which is called
        #           by the read method
        # setArgs:  A dictionary that specifies names and values of
        #           argumets to be passed to function
        # addArgs:  A list with the names of arguments to be passed
        #           to function. The requested data can only be
        #           retrieved from the dataset, if the values of
        #           these arguments are known at runtime.
        #           In contrast to setArgs, the values
        #           of these arguments cannot be specified during
        #           implementation time. Hence, the values must be passed to
        #           the read method.

        self._itemKeyToFuncArgsMap = {

            # Primary header keywords:

            "ExpTime":    {"function": self._readKeyword,
                           "setArgs": {"key": "EXPTIME"},
                           "addArgs": []},

            "Instrument": {"function": self._readKeyword,
                           "setArgs": {"key": "INSTRUME"},
                           "addArgs": []},

            "MBFitsVer":  {"function": self._readKeyword,
                           "setArgs": {"key": "MBFTSVER"},
                           "addArgs": []},

            "StartAz":    {"function": self._readKeyword,
                           "setArgs": {"key": "HIERARCH ESO TEL AZ"},
                           "addArgs": []},

            "StartAlt":   {"function": self._readKeyword,
                           "setArgs": {"key": "HIERARCH ESO TEL ALT"},
                           "addArgs": []},

            "StartRa":    {"function": self._readKeyword,
                           "setArgs": {"key": "RA"},
                           "addArgs": []},

            "StartDec":   {"function": self._readKeyword,
                           "setArgs": {"key": "DEC"},
                           "addArgs": []},



            # SCAN-MBFITS keywords:

            "Coord":   {"function": self._readKeywordTuple,
                          "setArgs": {"keys": ("CRVAL1", "CRVAL2"),
                          #"setArgs": {"keys": ("BLONGOBJ", "BLATOBJ"),
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Basis":   {"function": self._readKeywordTuple,
                          "setArgs": {"keys": ("CTYPE1", "CTYPE2"),
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Equinox":   {"function": self._readKeyword,
                          "setArgs": {"key": "EQUINOX",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "DateObs":   {"function": self._readKeyword,
                          "setArgs": {"key": "DATE-OBS",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Diameter":  {"function": self._readKeyword,
                          "setArgs": {"key": "DIAMETER",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanLST":   {"function": self._readKeyword,
                          "setArgs": {"key": "LST",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanMJD":   {"function": self._readKeyword,
                          "setArgs": {"key": "MJD",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "NObs":      {"function": self._readKeyword,
                          "setArgs": {"key": "NOBS",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "NSubs":     {"function": self._readKeyword,
                          "setArgs": {"key": "NSUBS",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Object":    {"function": self._readKeyword,
                          "setArgs": {"key": "OBJECT",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Observer":  {"function": self._readKeyword,
                          "setArgs": {"key": "OBSID",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "DeltaCA":   {"function": self._readKeyword,
                           "setArgs": {"key": "PDELTACA",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "DeltaIE":   {"function": self._readKeyword,
                           "setArgs": {"key": "PDELTAIE",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "Project":   {"function": self._readKeyword,
                           "setArgs": {"key": "PROJID",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "ScanGeom":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANGEOM",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanLen":   {"function": self._readKeyword,
                          "setArgs": {"key": "SCANLEN",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanLine":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANLINE",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanMode":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANMODE",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanNum":   {"function": self._readKeyword,
                          "setArgs": {"key": "SCANNUM",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanPar1":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANPAR1",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanPar2":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANPAR2",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanType":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANTYPE",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanXsp":   {"function": self._readKeyword,
                          "setArgs": {"key": "SCANXSPC",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanXVel":  {"function": self._readKeyword,
                          "setArgs": {"key": "SCANXVEL",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "ScanYsp":   {"function": self._readKeyword,
                          "setArgs": {"key": "SCANYSPC",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "SiteElev":  {"function": self._readKeyword,
                          "setArgs": {"key": "SITEELEV",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "SiteLat":   {"function": self._readKeyword,
                          "setArgs": {"key": "SITELAT",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "SiteLong":  {"function": self._readKeyword,
                          "setArgs": {"key": "SITELONG",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "Telescope": {"function": self._readKeyword,
                          "setArgs": {"key": "TELESCOP",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},

            "WobCycle":   {"function": self._readKeyword,
                           "setArgs": {"key": "WOBCYCLE",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "WobMode":    {"function": self._readKeyword,
                           "setArgs": {"key": "WOBMODE",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "WobThrow":   {"function": self._readKeyword,
                           "setArgs": {"key": "WOBTHROW",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "WobUsed":    {"function": self._readKeyword,
                           "setArgs": {"key": "WOBUSED",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "TAIUTC":     {"function": self._readKeyword,
                           "setArgs": {"key": "TAI2UTC",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},

            "UTCUT1":     {"function": self._readKeyword,
                           "setArgs": {"key": "UTC2UT1",
                                       "extname": "SCAN-MBFITS"},
                           "addArgs": []},


            # The following are used for writing .dat files used for
            # computing pointing model
            "PDeltaCA":    {"function": self._readKeyword,
                            "setArgs": {"key": "PDELTACA",
                                        "extname": "SCAN-MBFITS"},
                            "addArgs": []},
            "PDeltaIE":    {"function": self._readKeyword,
                            "setArgs": {"key": "PDELTAIE",
                                        "extname": "SCAN-MBFITS"},
                            "addArgs": []},
            "FDeltaCA":    {"function": self._readKeyword,
                            "setArgs": {"key": "FDELTACA",
                                        "extname": "SCAN-MBFITS"},
                            "addArgs": []},
            "FDeltaIE":    {"function": self._readKeyword,
                            "setArgs": {"key": "FDELTAIE",
                                        "extname": "SCAN-MBFITS"},
                            "addArgs": []},

            # SCAN-MBFITS columns:

            "Febes":     {"function": self._readColumn,
                          "setArgs": {"colname": "FEBE",
                                      "extname": "SCAN-MBFITS"},
                          "addArgs": []},




            # FEBEPAR-MBFITS keywords:

            "Colstart":  {"function": self._readKeyword,
                          "setArgs": {"key": "CARX",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "FdTypCod":  {"function": self._readKeyword,
                          "setArgs": {"key": "FDTYPCOD",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "Febe":      {"function": self._readKeyword,
                          "setArgs": {"key": "FEBE",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "FebeFeed":  {"function": self._readKeyword,
                          "setArgs": {"key": "FEBEFEED",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "Nula":      {"function": self._readKeyword,
                          "setArgs": {"key": "IARX",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "Nule":      {"function": self._readKeyword,
                          "setArgs": {"key": "IERX",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "FeedCode":  {"function": self._readKeyword,
                          "setArgs": {"key": "FDTYPCOD",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "DewCabin":  {"function": self._readKeyword,
                          "setArgs": {"key": "DEWCABIN",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},

            "DewUser":  {"function": self._readKeyword,
                          "setArgs": {"key": "DEWUSER",
                                      "extname": "FEBEPAR-MBFITS"},
                          "addArgs": ["febe"]},


            # FEBEPAR-MBFITS columns:

            "BolCalFc":  {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "BOLCALFC"},
                          "addArgs": ["febe"]},

           "FeedOffX":  {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "FEEDOFFX"},
                          "addArgs": ["febe"]},

            "FeedOffY":  {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "FEEDOFFY"},
                          "addArgs": ["febe"]},

            "FlatField": {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "FLATFIEL"},
                          "addArgs": ["febe"]},

            "NUseFeeds": {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "NUSEFEED"},
                          "addArgs": ["febe"]},

            "RefFeed":   {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "REFFEED"},
                          "addArgs": ["febe"]},

            "UseBand":   {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "USEBAND"},
                          "addArgs": ["febe"]},

            "UseFeed":   {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "USEFEED"},
                          "addArgs": ["febe"]},

            "FeedType":  {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "FEEDTYPE"},
                          "addArgs": ["febe"]},

            "DCoffset":  {"function": self._readColumnFebepar,
                          "setArgs": {"colname": "BOLDCOFF"},
                          "addArgs": ["febe"]},

            # DATAPAR-MBFITS keywords:

            "SubsStart": {"function": self._readKeyword,
                          "setArgs": {"key": "DATE-OBS",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "ObsNum":    {"function": self._readKeyword,
                          "setArgs": {"key": "OBSNUM",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "ObsType":   {"function": self._readKeyword,
                          "setArgs": {"key": "OBSTYPE",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "ObsStatus": {"function": self._readKeyword,
                          "setArgs": {"key": "OBSTATUS",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "DewExtra": {"function": self._readKeyword,
                          "setArgs": {"key": "DEWEXTRA",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            # DATAPAR-MBFITS columns:

            "Az":       {"function": self._readColumn,
                          "setArgs": {"colname": "AZIMUTH",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "BasLat":       {"function": self._readColumn,
                          "setArgs": {"colname": "BASLAT",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "BasLon":       {"function": self._readColumn,
                          "setArgs": {"colname": "BASLONG",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "El":       {"function": self._readColumn,
                          "setArgs": {"colname": "ELEVATIO",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "LatOff":   {"function": self._readColumn,
                          "setArgs": {"colname": "LATOFF",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "LonOff":    {"function": self._readColumn,
                          "setArgs": {"colname": "LONGOFF",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "LST":       {"function": self._readColumn,
                          "setArgs": {"colname": "LST",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "MJD":       {"function": self._readColumn,
                          "setArgs": {"colname": "MJD",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "LatPole":       {"function": self._readColumn,
                          "setArgs": {"colname": "MLATPOLE",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "LonPole":       {"function": self._readColumn,
                          "setArgs": {"colname": "MLONPOLE",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "RotAngle":  {"function": self._readColumn,
                          "setArgs": {"colname": "ROTANGLE",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "MVAL1":  {"function": self._readColumn,
                          "setArgs": {"colname": "MCRVAL1",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

            "MVAL2":  {"function": self._readColumn,
                          "setArgs": {"colname": "MCRVAL2",
                                      "extname": "DATAPAR-MBFITS"},
                          "addArgs": ["subsnum", "febe"]},

#             "WobblerSta": {"function": self._readColumn,
#                           "setArgs": {"colname": "PHASE",
#                                       "extname": "DATAPAR-MBFITS"},
#                           "addArgs": ["subsnum", "febe"]},



            # ARRAYDATA-MBFITS keywords:

            "RestFreq":  {"function": self._readKeyword,
                          "setArgs": {"key": "RESTFREQ",
                                      "extname": "ARRAYDATA-MBFITS"},
                          "addArgs": ["subsnum", "febe", "baseband"]},

            "NUseFeed":  {"function": self._readKeyword,
                          "setArgs": {"key": "NUSEFEED",
                                      "extname": "ARRAYDATA-MBFITS"},
                          "addArgs": ["subsnum", "febe", "baseband"]},

            # ARRAYDATA-MBFITS columns:

            "Data":      {"function": self._readData,
                          "setArgs": {},
                          "addArgs": ["subsnum", "febe", "baseband"]},


            # MONITOR-MBFITS keywords:

            "UsrFrame":  {"function": self._readKeyword,
                          "setArgs": {"key": "USRFRAME",
                                      "extname": "MONITOR-MBFITS"},
                          "addArgs": ["subsnum"]},

            # Custom:

            "FocX":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum", "febe"]},

            "FocY":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 1},
                          "addArgs": ["subsnum", "febe"]},

            "FocZ":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 2},
                          "addArgs": ["subsnum", "febe"]},

            "PhiX":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum", "febe"]},

            "PhiY":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 1},
                          "addArgs": ["subsnum", "febe"]},

            "PhiZ":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 2},
                          "addArgs": ["subsnum", "febe"]},

            "Subscans":  {"function": self._readSubscans,
                          "setArgs": {},
                          "addArgs": []},

            "NInteg":    {"function": self._readNInteg,
                          "setArgs": {},
                          "addArgs": ["subsnum", "febe", "baseband"]},

            "Refract":   {"function": self._readRefract,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum"]},

            "TempHe":   {"function": self._readHeTemp,
                          "setArgs": {},
                          "addArgs": ["subsnum"]},

            "WindSpeedDir": {"function": self._readWindSpeedDir,
                             "setArgs": {},
                             "addArgs": ["subsnum"]},

            "AzEl0":   {"function": self._readZeroPos,
                        "setArgs": {},
                        "addArgs": ["subsnum"]},

            "T_amb":   {"function": self._readTamb,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum"]},

            "BiasAmplitude": {"function": self._readBiasAmpl,
                              "setArgs": {"mpIndex": 0},
                              "addArgs": ["subsnum"]},

            "QdacAmplitude": {"function": self._readQdacAmpl,
                              "setArgs": {"mpIndex": 0},
                              "addArgs": ["subsnum"]},

            "BiasPotsetting": {"function": self._readBiasPotset,
                               "setArgs": {"mpIndex": 0},
                               "addArgs": ["subsnum"]},

            "GainSetting": {"function": self._readGainSetting,
                               "setArgs": {"mpIndex": 0},
                               "addArgs": ["subsnum"]},

            "PWV":            {"function": self._readPWV,
                               "setArgs": {},
                               "addArgs": ["subsnum"]},

            "WobblerSta": {"function": self._readWobblerSta,
                          "setArgs": {},
                          "addArgs": ["subsnum", "febe"]},

            "ScanDir":   {"function": self._readScanDir,
                          "setArgs": {},
                          "addArgs": ["subsnum", "febe"]},

            }

    # -----------------------------------------------------------------

    def _readColumnFebepar(self, colname, febe):
        # Read a column from a FEBEPAR table:

        # FEBEPAR tables are special in the following respects:
        # - They contain only one line; hence, it can be argued that the data from
        #   these columns should be arrays (or lists) of dimension 1 less than
        #   ordinary columns. This is implemented here.
        # - They contain a variaty of scalar, 1-dim, and 2-dim data, not allways
        #   with the correct TDIM keywords.

        # This make a seperate routine to read these columns necessary

        colsScalar = ["REFFEED"]

        try:
            tbl = self._getTable(extname="FEBEPAR-MBFITS", febe=febe)
            col = tbl.getColumn(colname)
            if not col:
                raise "Could not read column"

            rows = col.read()
            shape = col.getDim()

            row = rows[0]

            if colname in colsScalar:
                try:
                    result = row[0]
                except:
                    result = row
            else:
                if isinstance(row, type(np.array([]))):
                    result = row
                else:
                    try:
                        result = np.array([row])
                    except:
                        result = row

            if shape:
                try:
                    result.shape = shape
                except:
                    pass

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readData(self, subsnum, febe, baseband):
        # Special routine to read DATA column from ARRAYDATA table
        # Necessary since the ordering of data changed in May 2006
        try:

            nChannels = self._readKeyword("CHANNELS", \
                                          extname="ARRAYDATA-MBFITS", \
                                          subsnum=subsnum, \
                                          febe=febe, \
                                          baseband=baseband)

            data = self._readColumn(colname="DATA", \
                                    extname="ARRAYDATA-MBFITS", \
                                    subsnum=subsnum, \
                                    febe=febe, \
                                    baseband=baseband)

            if nChannels == data.shape[2]:
                # Since 2006/5/05 the spectral channels are the 3rd dim of data:
                nInteg, nUsfd, nCh = data.shape
                if nCh == 1:
                    result = data
                    result.shape = (nInteg, nUsfd)
                else:
                    # More than 1 spectral channel:
                    result = data[:,:, 0]
            else:
                # Before 2006/5/05 the spectral channels are the 2nd dim of data:
                nInteg, nCh, nUsfd = data.shape
                if nCh == 1:
                    result = data
                    result.shape = (nInteg, nUsfd)
                else:
                    # More than 1 spectral channel:
                    result = data[:, 0,:]

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDFocus(self, subsnum, febe, mpIndex):
        # Special routine to read Focus data necessary since
        # - data was originally stored in column DFOCUS in
        #   DATAPAR, but in columns DFOCUS_X, DFOCUS_Y, DFOCUS_Z
        #   later
        # - special case for FOCUS Scans: Here, the values from
        #   the MONITOR table are returned.
        try:
            # DFocusZ is used to decide whether the values
            # from DATAPAR or MONITOR is used:
            focusZ = self._readDFocusDatapar(subsnum=subsnum, \
                                             febe=febe, \
                                             mpIndex=2)

            # Read from DATAPAR:
            if mpIndex == 2:
                result = focusZ
            else:
                result = self._readDFocusDatapar(subsnum=subsnum, \
                                                 febe=febe, \
                                                 mpIndex=mpIndex)

            scanType = self.read("ScanType")
            if scanType.find('FOCUS') > -1:
                if focusZ[0] == self.getBlankFloat():
                    # Read first value from MONITOR and resize it to correct shape:
                    monFocusXYZ = self._readMonValue(monpoint="DFOCUS_X_Y_Z",
                                                     subsnum=subsnum)
                    monFocus = monFocusXYZ[0][mpIndex]
                    result = np.array([monFocus]*len(result),
                                           dtype=result.dtype)

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDFocusDatapar(self, subsnum, febe, mpIndex):
        try:
            tbls = self._dataset.getTables(EXTNAME="DATAPAR-MBFITS", \
                                           OBSNUM=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            if "DFOCUS" in tbls[0].getColumnNames():
                # Old format: All three values in single column:
                dfocus = self._readColumn(colname="DFOCUS", \
                                          extname="DATAPAR-MBFITS", \
                                          subsnum=subsnum, \
                                          febe=febe)
                try:
                    result = dfocus[:, mpIndex]
                except:
                    # If blanking value, the column contains only 1-dim data:
                    result = dfocus
            else:
                if mpIndex == 0:
                    result = self._readColumn(colname="DFOCUS_X", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
                elif mpIndex == 1:
                    result = self._readColumn(colname="DFOCUS_Y", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
                else:
                    result = self._readColumn(colname="DFOCUS_Z", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDPhi(self, subsnum, febe, mpIndex):
        # Special routine to read Phi data necessary since
        # - data was originally stored in column DPHI in
        #   DATAPAR, but in columns DPHI_X, DPHI_Y, DPHI_Z
        #   later
        # - special case for FOCUS Scans: Here, the values from
        #   the MONITOR table are returned.
        try:
            # DFocusZ is used to decide whether the values
            # from DATAPAR or MONITOR is used:
            focusZ = self._readDFocusDatapar(subsnum=subsnum, \
                                             febe=febe, \
                                             mpIndex=2)

            # Read from DATAPAR:
            result = self._readDPhiDatapar(subsnum=subsnum, \
                                           febe=febe, \
                                           mpIndex=mpIndex)

            scanType = self.read("ScanType")
            if scanType.find('FOCUS') > -1:
                if focusZ == self.getBlankFloat():
                    # Read frist value from MONITOR and resize it to correct shape:
                    monPhiXYZ = self._readMonValue(monpoint="DPHI_X_Y_Z",
                                                   subsnum=subsnum)
                    monPhi = monPhiXYZ[0][mpIndex]
                    result = np.array([monPhi]*len(result),
                                           dtype=result.dtype)

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDPhiDatapar(self, subsnum, febe, mpIndex):
        try:
            tbls = self._dataset.getTables(EXTNAME="DATAPAR-MBFITS", \
                                           OBSNUM=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            if "DPHI" in tbls[0].getColumnNames():
                # Old format: All three values in single column:
                dphi = self._readColumn(colname="DPHI", \
                                        extname="DATAPAR-MBFITS", \
                                        subsnum=subsnum, \
                                        febe=febe)
                try:
                    result = dphi[:, mpIndex]
                except:
                    # If blanking value, the column contains only 1-dim data:
                    result = dphi
            else:
                if mpIndex == 0:
                    result = self._readColumn(colname="DPHI_X", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
                elif mpIndex == 1:
                    result = self._readColumn(colname="DPHI_Y", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
                else:
                    result = self._readColumn(colname="DPHI_Z", \
                                              extname="DATAPAR-MBFITS", \
                                              subsnum=subsnum, \
                                              febe=febe)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readWobblerSta(self, subsnum, febe):
        # Special routine necessary since data was originally
        # stored in column ISWITCH in DATAPAR, while later
        # column PHASE was used.
        # In the latter case, PHASE must be translated to the corresponding
        # string.
        try:
            tbls = self._dataset.getTables(EXTNAME="DATAPAR-MBFITS", \
                                           OBSNUM=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            if "ISWITCH" in tbls[0].getColumnNames():
                # Old format: String in column:
                result = self._readColumn(colname="ISWITCH", \
                                          extname="DATAPAR-MBFITS", \
                                          subsnum=subsnum, \
                                          febe=febe)
                for i in np.arange(len(result)):
                    if result[i] == "NONE":
                        result[i] = ""
            else:
                # New format: Phase number in column:
                phases = self._readColumn(colname="PHASE", \
                                          extname="DATAPAR-MBFITS", \
                                          subsnum=subsnum, \
                                          febe=febe)
                # Need to read the phase description (which one is ON,
                # which one is OFF). Defined in SCAN header, or re-defined
                # in DATAPAR header if different for that specific subscan
                # Try first to get it from DATAPAR
                phas1 = tbls[0].getKeyword('PHASE1')
                phas2 = tbls[0].getKeyword('PHASE2')
                if (not phas1) or (not phas2):
                    # at least one is missing, search in SCAN header
                    scantbl = self._dataset.getTables(EXTNAME="SCAN-MBFITS")
                    phas1 = scantbl[0].getKeyword('PHASE1')
                    phas2 = scantbl[0].getKeyword('PHASE2')

                phas1 = phas1.getValue()
                phas2 = phas2.getValue()

                result = []
                thePhases = []
                if phas1 == 'WON':
                    thePhases.append('ON')
                elif phas1 == 'WOFF':
                    thePhases.append('OFF')
                else:
                    thePhases.append('')
                if phas2 == 'WON':
                    thePhases.append('ON')
                elif phas2 == 'WOFF':
                    thePhases.append('OFF')
                else:
                    thePhases.append('')

                # Translate:
                for phase in phases:
                    if phase == self.getBlankInt():
                        result.append("")
                    elif phase == 1 or phase == 2:
                        result.append(thePhases[phase-1])
                    else:
                        result.append('')

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------
    def _readScanDir(self, subsnum, febe):
        # Another special case where the information can be either
        # in the SCAN header, or in DATAPAR headers if it changes
        # from subscan to subscan
        try:
            tbls = self._dataset.getTables(EXTNAME="DATAPAR-MBFITS", \
                                           OBSNUM=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            # Try first to get values from DATAPAR
            onedir = tbls[0].getKeyword('SCANDIR')
            if not onedir:
                # search in SCAN header
                scantbl = self._dataset.getTables(EXTNAME="SCAN-MBFITS")
                onedir = scantbl[0].getKeyword('SCANDIR')

            return onedir.getValue()
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readMonValue(self, subsnum, monpoint,getTime = 0):
        try:
            tbls = self._dataset.getTables(EXTNAME="MONITOR-MBFITS", \
                                           OBSNUM=subsnum)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"
            col = tbls[0].getColumn("MONVALUE")
            if not col:
                raise "Could not read column"
            if getTime:
                colTime = tbls[0].getColumn("MJD")
            tbls[0].setSelection("MONPOINT=='%s'" % monpoint)
            result = col.read()
            if getTime:
                resTime = colTime.read()
                result = (result, resTime)
            tbls[0].clearSelection()

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readSubscans(self):
        # Subscan numbers from MONITOR tables
        try:
            subscans = []
            tbls = self._dataset.getTables(EXTNAME="MONITOR-MBFITS")
            for tbl in tbls:
                # Use OBSNUM instead of SUBSNUM, since some files have
                # SUBSNUM=0 for all subscans
                kw = tbl.getKeyword("OBSNUM")
                if not kw:
                    raise "Could not read keyword 'OBSNUM'"
                subscans.append(kw.getValue())
            return subscans
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readNInteg(self, subsnum, febe, baseband):
        # Number of integrations from ARRAYDATA table
        try:
            tbls = self._dataset.getTables(EXTNAME="ARRAYDATA-MBFITS", \
                                           OBSNUM=subsnum,
                                           FEBE=febe,
                                           BASEBAND=baseband)
            if len(tbls) == 0:
                msg  = "No table selected for "
                msg += "EXTNAME=%s " % str("ARRAYDATA-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                msg += "FEBE=%s " % str(febe)
                msg += "BASEBAND=%s " % str(baseband)
                raise msg
            elif len(tbls) > 1:
                msg  = "No unique table selected for "
                msg += "EXTNAME=%s " % str("ARRAYDATA-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                msg += "FEBE=%s " % str(febe)
                msg += "BASEBAND=%s " % str(baseband)
                raise msg

            nbRowsArraydata = tbls[0].getNumRows()

            tbls = self._dataset.getTables(EXTNAME="DATAPAR-MBFITS", \
                                           OBSNUM=subsnum,
                                           FEBE=febe)
            if len(tbls) == 0:
                msg  = "No table selected for "
                msg += "EXTNAME=%s " % str("DATAPAR-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                msg += "FEBE=%s " % str(febe)
                raise msg
            elif len(tbls) > 1:
                msg  = "No unique table selected for "
                msg += "EXTNAME=%s " % str("DATAPAR-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                msg += "FEBE=%s " % str(febe)
                raise msg

            nbRowsDatapar = tbls[0].getNumRows()

            result = min((nbRowsArraydata, nbRowsDatapar))
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readRefract(self, subsnum, mpIndex):
        try:
            monRefrac = self._readMonValue(monpoint="REFRACTION",
                                           subsnum=subsnum)
            monRefrac = monRefrac[0][mpIndex]
            result = np.array([monRefrac],
                                   dtype=np.float32)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readTamb(self, subsnum, mpIndex):
        try:
            monTambPHum = self._readMonValue(monpoint="TAMB_P_HUMID",
                                             subsnum=subsnum)
            monTamb = monTambPHum[0][mpIndex]
            result = np.array([monTamb], 'f')
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readHeTemp(self, subsnum):
        try:
            monHeT, timeHe = self._readMonValue(monpoint="LABOCA_HE3TEMP",\
                                               subsnum=subsnum,\
                                               getTime=1)
            result = []
            times = []
            nb = len(timeHe)
            for i in range(nb):
                if monHeT[i][0] != self.getBlankFloat() and \
                       timeHe[i] != self.getBlankFloat():
                    result.append(monHeT[i][0])
                    # 32s shift in the timestamps!
                    times.append(timeHe[i]-32./86400.)
                    # TODO: remove this -32./86400. when fixed by Giorgio

            times  = np.array((times), 'd')
            result = np.array((result), 'f')
            return result, times
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readWindSpeedDir(self, subsnum):
        try:
            windSpeedDir, timeWind = self._readMonValue(monpoint="WIND_SPEED_DIR",\
                                                       subsnum=subsnum,\
                                                       getTime=1)


            aWindSpeedDir = np.array(windSpeedDir)
            return aWindSpeedDir[:, 0], aWindSpeedDir[:, 1], np.array(timeWind)
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readGains(self, febe):

        tbl = self._getTable(extname="FEBEPAR-MBFITS", febe=febe)

        # Frontend Gain
        kwFeGain = tbl.getKeyword('FEGAIN')
        if kwFeGain:
            kwFeGain = kwFeGain.getValue()
        else:
            kwFeGain = 0.

        # Backend Gain
        try:
            colBeGain = self._readColumnFebepar(colname="BEGAIN", \
                                                febe=febe)
        except MBFitsReaderError:
            colBeGain = self._readColumnFebepar(colname="BOLREFGN", \
                                                febe=febe)
        if not colBeGain:
            raise "Could not read keyword BE Gain"

        if kwFeGain == self.getBlankFloat():
            kwFeGain = 0.
        if colBeGain == self.getBlankFloat():
            colBeGain = 1.

        return kwFeGain, colBeGain

    # -----------------------------------------------------------------
    def _readZeroPos(self, subsnum):

        """
        Read and return ANTENNA and ENCODE Az, El at scan start
        """
        # try:
        monEncoder = self._readMonValue(monpoint="ENCODER_AZ_EL",\
                                        subsnum=subsnum)
        monAntenna = self._readMonValue(monpoint="ANTENNA_AZ_EL",\
                                        subsnum=subsnum)
        return monAntenna[0], monEncoder[0]
        # except Exception, data:
        #    raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readBiasAmpl(self, subsnum, mpIndex):
        try:
            monBiasAmpl = self._readMonValue(monpoint="BOLOSZ_BIASAMPLITUDE",
                                             subsnum=subsnum)
            if monBiasAmpl:
                result = monBiasAmpl[mpIndex]
            else:
                result = None
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readQdacAmpl(self, subsnum, mpIndex):
        try:
            monQdacAmpl = self._readMonValue(monpoint="BOLOSZ_QDACAMPLITUDE",
                                             subsnum=subsnum)
            if monQdacAmpl:
                result = monQdacAmpl[mpIndex]
            else:
                result = None
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))


    # -----------------------------------------------------------------

    def _readBiasPotset(self, subsnum, mpIndex):
        try:
            monBiasPotset = self._readMonValue(monpoint="BOLOSZ_BIASPOTSETTING",
                                               subsnum=subsnum)
            if monBiasPotset:
                result = monBiasPotset[mpIndex]
            else:
                result = None
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readGainSetting(self, subsnum, mpIndex):
        try:
            monGainSetting = self._readMonValue(monpoint="BOLOSZ_DEMODATTEN",
                                               subsnum=subsnum)
            if monGainSetting:
                result = monGainSetting[mpIndex]
            else:
                result = None
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readPWV(self, subsnum):
        try:
            monPWV = self._readMonValue(monpoint="PWV",
                                        subsnum=subsnum)
            return monPWV
        except Exception as data:
            raise MBFitsReaderError(str(data))

# ---------------------------------------------------------------------
# ---- Class IramMBFitsReader -----------------------------------------
# ---------------------------------------------------------------------

class IramMBFitsReader(MBFitsReader):
    """
    DES: Reader class for IRAM-MBFITS
         Consult the documentation of the superclass MBFitsReader
         and the source code of the init method to find out what the
         class does.
    """

    def __init__(self, dataset):
        MBFitsReader.__init__(self, dataset)

        # Two dictionary used to cache data for the interpolations:
        self._MJD = {} # key: subscan number; item: MJDs from IMBF-backend
        self._MJD_dap_s = {} # key: (subscan, febe); item: MJDs from IMBF-antenna
        self._MJD_dap_f = {} # key: (subscan, febe); item: MJDs from IMBF-antenna

        # The following dictionary maps each itemKey to
        # a method of MBFitsReader together with information
        # about arguments of the call to this method.

        # Each entry into the dictionary must be of the form:
        # itemKey: {"function": function,
        #           "setArgs": setArgsDict,
        #           "addArgs": addArgsList}

        # itemKey:  The key to be used in the read method
        # function: The method of MBFitsReader which is called
        #           by the read method
        # setArgs:  A dictionary that specifies names and values of
        #           argumets to be passed to function
        # addArgs:  A list with the names of arguments to be passed
        #           to function. The requested data can only be
        #           retrieved from the dataset, if the values of
        #           these arguments are known at runtime.
        #           In contrast to setArgs, the values
        #           of these arguments cannot be specified during
        #           implementation time. Hence, the values must be passed to
        #           the read method.

        self._itemKeyToFuncArgsMap = {

            # Primary header keywords:

            "ExpTime":    {"function": self._readKeyword,
                           "setArgs": {"key": "EXPTIME"},
                           "addArgs": []},

            "Instrument": {"function": self._readKeyword,
                           "setArgs": {"key": "INSTRUME"},
                           "addArgs": []},

            "MBFitsVer":  {"function": self._readKeyword,
                           "setArgs": {"key": "IMBFTSVE"},
                           "addArgs": []},

            "DateObs":   {"function": self._readKeyword,
                          "setArgs": {"key": "DATE-OBS",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "ScanLST":   {"function": self._readKeyword,
                          "setArgs": {"key": "LST",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "ScanMJD":   {"function": self._readKeyword,
                          "setArgs": {"key": "MJD",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "NObs":      {"function": self._readKeyword,
                          "setArgs": {"key": "N_OBS",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "NSubs":     {"function": self._readKeyword,
                          "setArgs": {"key": "N_OBSP",
                                      "extname": "IMBF-scan"},

                          "addArgs": []},

            "Object":    {"function": self._readKeyword,
                          "setArgs": {"key": "OBJECT",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "Telescope": {"function": self._readKeyword,
                          "setArgs": {"key": "TELESCOP",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "ScanType":  {"function": self._readKeyword,
                          "setArgs": {"key": "OBSTYPE",
                                      "extname": "IMBF-antenna-s",
                                      "subsnum": 1},
                          "addArgs": []},

            # IMBF-scan keywords:

            "Coord":   {"function": self._readKeywordTuple,
                          "setArgs": {"keys": ("LONGOBJ", "LATOBJ"),
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "Basis":   {"function": self._readKeywordTuple,
                          "setArgs": {"keys": ("CTYPE1", "CTYPE2"),
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "Equinox":   {"function": self._readKeyword,
                          "setArgs": {"key": "EQUINOX",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "Diameter":  {"function": self._readKeyword,
                          "setArgs": {"key": "TELSIZE",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "Observer":  {"function": self._readKeyword,
                          "setArgs": {"key": "OBSID",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "DeltaCA":   {"function": self._readKeyword,
                           "setArgs": {"key": "P2COR",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "DeltaIE":   {"function": self._readKeyword,
                           "setArgs": {"key": "P7COR",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "TAIUTC":     {"function": self._readKeyword,
                           "setArgs": {"key": "TAIUTC",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},
            # NIKA : wrong definition
            "UTCUT1":     {"function": self._readKeyword,
                           "setArgs": {"key": "UT1UTC",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "Project":   {"function": self._readKeyword,
                           "setArgs": {"key": "PROJID",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "ScanNum":   {"function": self._readKeyword,
                          "setArgs": {"key": "SCANNUM",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "SiteElev":  {"function": self._readKeyword,
                          "setArgs": {"key": "SITEELEV",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "SiteLat":   {"function": self._readKeyword,
                          "setArgs": {"key": "SITELAT",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "SiteLong":  {"function": self._readKeyword,
                          "setArgs": {"key": "SITELONG",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},

            "WobCycle":   {"function": self._readKeyword,
                           "setArgs": {"key": "WOBCYCLE",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "WobMode":    {"function": self._readKeyword,
                           "setArgs": {"key": "WOBMODE",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "WobThrow":   {"function": self._readKeyword,
                           "setArgs": {"key": "WOBTHROW",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "WobUsed":    {"function": self._readKeyword,
                           "setArgs": {"key": "WOBUSED",
                                       "extname": "IMBF-scan"},
                           "addArgs": []},

            "RefFeed":   {"function": self._readKeyword,
                          "setArgs": {"key": "RECON",
                                      "extname": "IMBF-scan"},
                          "addArgs": []},


            # IMBF-scan columns:
            # nothing in the IMBF-scan table... take instrument from Primary

            "Febes":     {"function": self._readFebes,
                           "setArgs": {},
                           "addArgs": []},



            # IMBF-frontend keywords:

            "Febe":      {"function": self._readKeyword,
                          "setArgs": {"key": "FEBE",
                                      "extname": "IMBF-frontend"},
                          "addArgs": ["febe"]},

            "FebeFeed":  {"function": self._readKeyword,
                          "setArgs": {"key": "FEBEFEED",
                                      "extname": "IMBF-frontend"},
                          "addArgs": []},

            "NUseFeeds": {"function": self._readKeywordFebe,
                          "setArgs": {"key": "NUSEFEED",
                                      "extname": "IMBF-frontend"},
                          "addArgs": []},

            "DewCabin":  {"function": self._readKeyword,
                          "setArgs": {"key": "DEWCABIN",
                                      "extname": "IMBF-frontend"},
                          "addArgs": []},

            # IMBF-frontend columns:

            "UseFeed":   {"function": self._readColumnNIKA,
                          "setArgs": {"colname": "USEFEED",
                                      "extname": "IMBF-backend"},
                          "addArgs": []},

            "FeedType":  {"function": self._readColumnNIKA,
                          "setArgs": {"colname": "FEEDTYPE",
                                      "extname": "IMBF-backend"},
                          "addArgs": []},

            "FeedOffX":  {"function": self._readColumn,
                          "setArgs": {"colname": "NAS_X",
                                      "extname": "Params"},
                          "addArgs": []},

            "FeedOffY":  {"function": self._readColumn,
                          "setArgs": {"colname": "NAS_Y",
                                      "extname": "Params"},
                          "addArgs": []},

            "FlatField":  {"function": self._readColumnNIKA,
                          "setArgs": {"colname": "FLATFIELD",
                                      "extname": "IMBF-backend"},
                          "addArgs": []},

           "DCoffset":  {"function": self._readColumnNIKA,
                          "setArgs": {"colname": "DCOFFSET",
                                      "extname": "IMBF-backend"},
                          "addArgs": []},

            "RestFreq":  {"function": self._readColumnFrontend,
                          "setArgs": {"colname": "RESTFREQ"},
                          "addArgs": ["febe"]},

            # IMBF-antenna keywords:

            "ScanMode":  {"function": self._readKeyword,
                          "setArgs": {"key": "SUBSTYPE",
                                      "extname": "IMBF-antenna-f",
                                      "subsnum": 1},
                          "addArgs": []},

            "SubsStart": {"function": self._readKeyword,
                          "setArgs": {"key": "DATE-OBS",
                                      "extname": "IMBF-antenna-f"},
                          "addArgs": ["subsnum"]},

            "ObsNum":    {"function": self._readKeyword,
                          "setArgs": {"key": "OBSNUM",
                                      "extname": "IMBF-antenna-f"},
                          "addArgs": ["subsnum"]},

            "ObsType":   {"function": self._readKeyword,
                          "setArgs": {"key": "OBSTYPE",
                                      "extname": "IMBF-antenna-f"},
                          "addArgs": ["subsnum"]},

            # IMBF-antenna columns:

            "Az":       {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "AZIMUTH"},
                          "addArgs": ["subsnum", "febe"]},

            "BasLat":       {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "BASLAT"},
                          "addArgs": ["subsnum", "febe"]},

            "BasLon":       {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "BASLONG"},
                          "addArgs": ["subsnum", "febe"]},

            "El":       {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "ELEVATION"},
                          "addArgs": ["subsnum", "febe"]},

            "LatOff":   {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "LATOFF"},
                          "addArgs": ["subsnum", "febe"]},

            "LonOff":    {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "LONGOFF"},
                          "addArgs": ["subsnum", "febe"]},

            "LST":       {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "LST"},
                          "addArgs": ["subsnum", "febe"]},

            "MJD-dap":   {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "MJD"},
                          "addArgs": ["subsnum", "febe"]},

            "RotAngle":  {"function": self._readColumnAntenna,
                          "setArgs": {"colname": "PARANGLE"},
                          "addArgs": ["subsnum", "febe"]},


            # ARRAYDATA-MBFITS keywords:

            # "NUseFeed":  {"function": self._readKeywordBackend,
            #               "setArgs": {"key": "CHANNELS"},
            #               "addArgs": ["subsnum", "febe"]},

            "NUseFeed":  {"function": self._readNUseFeedNIKA,
                          "setArgs": {"key": "CHANNELS"},
                          "addArgs": ["subsnum", "febe"]},

            # IMBF-backend columns:

            # "Data":      {"function": self._readColumnBackend,
            #               "setArgs": {"colname": "DATA"},
            #               "addArgs": ["subsnum", "febe"]},
            # NIKA : This is uncalibrated data
            "Data":      {"function": self._readColumnBackend,
                          "setArgs": {"colname": "R"},
                          "addArgs": ["subsnum", "febe"]},

            "MJD":       {"function": self._readColumnBackend,
                          "setArgs": {"colname": "MJD"},
                          "addArgs": ["subsnum", "febe"]},

            "WobblerSta": {"function": self._readColumnBackend,
                          "setArgs": {"colname": "ISWITCH"},
                          "addArgs": ["subsnum", "febe"]},

            # Custom:

            "FocX":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum", "febe"]},

            "FocY":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 1},
                          "addArgs": ["subsnum", "febe"]},

            "FocZ":      {"function": self._readDFocus,
                          "setArgs": {"mpIndex": 2},
                          "addArgs": ["subsnum", "febe"]},

            "PhiX":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum", "febe"]},

            "PhiY":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 1},
                          "addArgs": ["subsnum", "febe"]},

            "PhiZ":      {"function": self._readDPhi,
                          "setArgs": {"mpIndex": 2},
                          "addArgs": ["subsnum", "febe"]},

            "Subscans":  {"function": self._readSubscans,
                          "setArgs": {},
                          "addArgs": []},

            "NInteg":    {"function": self._readNInteg,
                          "setArgs": {},
                          "addArgs": ["subsnum", "febe", "baseband"]},

            "Refract":   {"function": self._readRefract,
                          "setArgs": {"mpIndex": 0},
                          "addArgs": ["subsnum"]},

            # Some items not relevant for IRAM data
            "UseBand":   {"function": self._readUseBand,
                          "setArgs": {},
                          "addArgs": []},

            "FeedCode":  {"function": self._readFeedcode,
                          "setArgs": {},
                          "addArgs": []},

            "BolCalFc":  {"function": self._readBolCal,
                          "setArgs": {},
                          "addArgs": []},

            "UsrFrame":  {"function": self._readUsrFrame,
                          "setArgs": { "extname": "IMBF-antenna-s"},
                          "addArgs": ["subsnum"]},

            }


    # -----------------------------------------------------------------

    def _readFebes(self):
        # Take instrument from Primary:
        febe = self._readKeyword("INSTRUME")
        return [febe]

    # -----------------------------------------------------------------
    # WARN : NIKA trik because IMBFITS are really bad...
    #
    def _readColumnNIKA(self, colname, extname, **kargs):
        # Read a column:
        try:
            tbl = self._getTable(extname=extname, **kargs)
            if colname == "FEEDTYPE":
                result = np.repeat(1, tbl.getKeyword('NUSEFEED').getValue())[np.newaxis,:]
            elif colname == "DCOFFSET" or colname == "FLATFIELD":
                result = np.repeat(0, tbl.getKeyword('NUSEFEED').getValue())[np.newaxis,:]

            else:
                col = tbl.getColumn(colname)
                if not col:
                    raise "Could not read column"
                result = col.read()
                # TODO :  IMBFITS format have TFORM = 168J instead of 1PJ(168)"
                result = result[:,:, 0]

            return result

        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readColumnAntenna(self, colname, subsnum, febe):
        # Fill the MJDs for the interpolation if not allready present:
        if not (subsnum in self._MJD_dap_f.keys()):
            tmpArray = self._readColumn(colname="MJDFAST", \
                                            extname="IMBF-antenna-f", \
                                            subsnum=subsnum)
            self._MJD_dap_f[subsnum] = tmpArray[:, 0, 0]

        if not (subsnum in self._MJD_dap_s.keys()):
            self._MJD_dap_s[subsnum] = self._readColumn(colname="MJD", \
                                                            extname="IMBF-antenna-s", \
                                                            subsnum=subsnum)

        if not ((subsnum, febe) in self._MJD.keys()):
            self._MJD[(subsnum, febe)] = self.read("MJD", \
                                                   subsnum=subsnum, \
                                                   febe=febe).astype(np.float)

        if colname in ["LST", "LONGOFF", "LATOFF", "CAZIMUTH", "CELEVATIO", "BASLONG", "BASLAT", "PARANGLE"]:
            extname = "IMBF-antenna-s"
            MJD_dap = self._MJD_dap_s[subsnum]
        elif colname in ["MJDFAST", "AZIMUTH", "TRACKING_AZ", "ELEVATION", "TRACKING_EL", "AZTRACKFLAG", "ELTRACKFLAG"]:
            extname = "IMBF-antenna-f"
            MJD_dap = self._MJD_dap_f[subsnum]

        tmpArray = self._readColumn(colname=colname, \
                                    extname=extname, \
                                    subsnum=subsnum)

        if colname in ("AZIMUTH", "ELEVATION", "TRACKING_AZ", "TRACKING_EL"):
            tmpArray = tmpArray[:, 0, 0]


        tmpInterp = np.interp(self._MJD[(subsnum, febe)], MJD_dap, tmpArray)

        if colname in ("AZIMUTH", "CAZIMUTH", "TRACKING_AZ", "ELEVATION", "CELEVATIO", "TRACKING_EL", "LONGOFF", "LATOFF", "BASLONG", "BASLAT", "PARANGLE"):
            tmpInterp *= 180./np.pi

        return tmpInterp

    # -----------------------------------------------------------------


    def _readColumnFrontend(self, colname, febe):
        # Read a column from the IMBF-frontend table:

        # The IMBF-frontend table is special in the following respects:
        # - It contains only one line; hence, it can be argued that the data from
        #   these columns should be arrays (or lists) of dimension 1 less than
        #   ordinary columns. This is implemented here.
        # - It contains a variaty of scalar, 1-dim, and 2-dim data, not allways
        #   with the correct TDIM keywords.

        # This make a seperate routine to read these columns necessary

        colsScalar = ["RESTFREQ"]

        try:
#            tbl = self._getTable(extname="IMBF-frontend", febe=febe)
            tbl = self._getTable(extname="IMBF-frontend")
            col = tbl.getColumn(colname)
            if not col:
                raise "Could not read column"

            rows = col.read()
            row = rows[0]

            if colname in colsScalar:
                result = row[0][0]
            else:
                if isinstance(row, type(np.array([]))):
                    result = [row[:, 0]]
                else:
                    result = [np.array(row)]

            if colname == "RESTFREQ":
                # Scale from GHz to Hz
                result *= 1.e9

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------
    # IMBF-backendXXX table :
    # need special routines because table name depends on backend
    def _readKeywordBackend(self, key, subsnum, febe):
        if '-' in febe:
            tmp = febe.split('-')
            be = tmp[1]
        else:
            be = febe

        tabName = "IMBF-backend"+be
        result = self._readKeyword(key, extname=tabName, subsnum=subsnum)
        return result

    def _readColumnBackend(self, colname, subsnum, febe):
        if '-' in febe:
            tmp = febe.split('-')
            be = tmp[1]
        else:
            be = febe

        tabName = "IMBF-backend"+be

        try:
            tbl = self._getTable(extname=tabName, subsnum=subsnum)
            col = tbl.getColumn(colname)
            if not col:
                raise "Could not read column"

            rows = col.read()
            shape = col.getDim()

            if isinstance(rows, type(np.array([]))):
                result = rows
            else:
                try:
                    result = np.array(rows)
                except:
                    result = rows

            # NIKA Trick
            if colname == "DATA" or colname == "R":
                result = result[:,:, 0]

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDFocus(self, subsnum, febe, mpIndex):
        # Special routine to read Focus data necessary since
        # - data was originally stored in column DFOCUS in
        #   DATAPAR, but in columns DFOCUS_X, DFOCUS_Y, DFOCUS_Z
        #   later
        # - special case for FOCUS Scans: Here, the values from
        #   the MONITOR table are returned.
        try:
            # DFocusZ is used to decide whether the values
            # from DATAPAR or MONITOR is used:
            focusZ = self._readDFocusDatapar(subsnum=subsnum, \
                                             febe=febe, \
                                             mpIndex=2)

            # Read from DATAPAR:
            if mpIndex == 2:
                result = focusZ
            else:
                result = self._readDFocusDatapar(subsnum=subsnum, \
                                                 febe=febe, \
                                                 mpIndex=mpIndex)

            scanType = self.read("ScanType")
            if scanType.find('FOCUS') > -1:
                if focusZ[0] == self.getBlankFloat():
                    # Read frist value from MONITOR and resize it to correct shape:
                    monFocusXYZ = self._readMonValue(monpoint="DFOCUS_X_Y_Z",
                                                     subsnum=subsnum)
                    monFocus = monFocusXYZ[0][mpIndex]
                    result = np.array([monFocus]*len(result),
                                           dtype=result.dtype)

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDFocusDatapar(self, subsnum, febe, mpIndex):
        try:
            tbls = self._dataset.getTables(EXTNAME="IMBF-antenna", \
                                           OBSNU=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            if "DFOCUS" in tbls[0].getColumnNames():
                # Old format: All three values in single column:
                dfocus = self._readColumn(colname="DFOCUS", \
                                          extname="IMBF-antenna", \
                                          subsnum=subsnum, \
                                          febe=febe)
                try:
                    result = dfocus[:, mpIndex]
                except:
                    # If blanking value, the column contains only 1-dim data:
                    result = dfocus
            else:
                if mpIndex == 0:
                    result = self._readColumn(colname="DFOCUS_X", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
                elif mpIndex == 1:
                    result = self._readColumn(colname="DFOCUS_Y", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
                else:
                    result = self._readColumn(colname="DFOCUS_Z", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDPhi(self, subsnum, febe, mpIndex):
        # Special routine to read Phi data necessary since
        # - data was originally stored in column DPHI in
        #   DATAPAR, but in columns DPHI_X, DPHI_Y, DPHI_Z
        #   later
        # - special case for FOCUS Scans: Here, the values from
        #   the MONITOR table are returned.
        try:
            # DFocusZ is used to decide whether the values
            # from DATAPAR or MONITOR is used:
            focusZ = self._readDFocusDatapar(subsnum=subsnum, \
                                             febe=febe, \
                                             mpIndex=2)

            # Read from DATAPAR:
            result = self._readDPhiDatapar(subsnum=subsnum, \
                                           febe=febe, \
                                           mpIndex=mpIndex)

            scanType = self.read("ScanType")
            if scanType.find('FOCUS') > -1:
                if focusZ == self.getBlankFloat():
                    # Read frist value from MONITOR and resize it to correct shape:
                    monPhiXYZ = self._readMonValue(monpoint="DPHI_X_Y_Z",
                                                   subsnum=subsnum)
                    monPhi = monPhiXYZ[0][mpIndex]
                    result = np.array([monPhi]*len(result),
                                           dtype=result.dtype)

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readDPhiDatapar(self, subsnum, febe, mpIndex):
        try:
            tbls = self._dataset.getTables(EXTNAME="IMBF-antenna", \
                                           OBSNUM=subsnum, \
                                           FEBE=febe)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"

            if "DPHI" in tbls[0].getColumnNames():
                # Old format: All three values in single column:
                dphi = self._readColumn(colname="DPHI", \
                                        extname="IMBF-antenna", \
                                        subsnum=subsnum, \
                                        febe=febe)
                try:
                    result = dphi[:, mpIndex]
                except:
                    # If blanking value, the column contains only 1-dim data:
                    result = dphi
            else:
                if mpIndex == 0:
                    result = self._readColumn(colname="DPHI_X", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
                elif mpIndex == 1:
                    result = self._readColumn(colname="DPHI_Y", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
                else:
                    result = self._readColumn(colname="DPHI_Z", \
                                              extname="IMBF-antenna", \
                                              subsnum=subsnum, \
                                              febe=febe)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------
    def _readMonValue(self, subsnum, monpoint):
        try:
            tbls = self._dataset.getTables(EXTNAME="MONITOR-MBFITS", \
                                           OBSNUM=subsnum)
            if len(tbls) == 0:
                raise "No table selected"
            elif len(tbls) > 1:
                raise "No unique table selected"
            col = tbls[0].getColumn("MONVALUE")
            if not col:
                raise "Could not read column"
            tbls[0].setSelection("MONPOINT=='%s'" % monpoint)
            result = col.read()
            tbls[0].clearSelection()

            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readSubscans(self):
        # Subscan numbers from IMBF-antenna tables
        try:
            subscans = []
            tbls = self._dataset.getTables(EXTNAME="IMBF-antenna")
            # TODO: This only works for IMBFITS v.. ?, silly code...
            # TODO: ugly workaround for the moment
            if tbls == [] :
                tbls = self._dataset.getTables(EXTNAME="IMBF-antenna-s")
            for tbl in tbls:
                kw = tbl.getKeyword("OBSNUM")
                if not kw:
                    raise "Could not read keyword 'OBSNUM'"
                subscans.append(kw.getValue())
            return subscans
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readNInteg(self, subsnum, febe, baseband):
        # Number of integrations from backend table
        # split FEBE name to retrieve backend
        if '-' in febe:
            tmp = febe.split('-')
            be = tmp[1]
        else:
            be = febe

        tabName = "IMBF-backend"+be
        try:
            tbls = self._dataset.getTables(EXTNAME=tabName, \
                                           OBSNUM=subsnum)
            if len(tbls) == 0:
                msg  = "No table selected for "
                msg += "EXTNAME=%s " % str("ARRAYDATA-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                raise MBFitsReaderError(msg)
            elif len(tbls) > 1:
                msg  = "No unique table selected for "
                msg += "EXTNAME=%s " % str("ARRAYDATA-MBFITS")
                msg += "OBSNUM=%s " % str(subsnum)
                raise MBFitsReaderError(msg)
            result = tbls[0].getNumRows()
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readRefract(self, subsnum, mpIndex):
        try:
            monRefrac = self._readMonValue(monpoint="REFRACTION",
                                           subsnum=subsnum)
            monRefrac = monRefrac[0][mpIndex]
            result = np.array([monRefrac],
                                   dtype=np.float32)
            return result
        except Exception as data:
            raise MBFitsReaderError(str(data))

    # -----------------------------------------------------------------

    def _readGains(self, febe):
        # split FEBE name to retrieve backend
        if '-' in febe:
            tmp = febe.split('-')
            be = tmp[1]
        else:
            be = febe

        tabName = "IMBF-backend"+be

        tbls = self._dataset.getTables(EXTNAME=tabName)
        if len(tbls) == 0:
            raise "No table selected"
        elif len(tbls) >= 1:
            tbls = tbls[0]

        # No Frontend Gain for MAMBO
        kwFeGain = 0.

        # Backend Gain
        kwBeGain = tbls.getKeyword('BOLGAIN')
        if kwBeGain:
            kwBeGain = kwBeGain.getValue()
        else:
            kwBeGain = 0.
        if not kwBeGain:
            kwBeGain = 1.
            # NIKA Trick
            # raise "Could not read keyword BE Gain"

        if kwFeGain == self.getBlankFloat():
            kwFeGain = 0.
        if kwBeGain == self.getBlankFloat():
            kwBeGain = 1.

        return kwFeGain, kwBeGain

    def _readUseBand(self):
        # Only one band in MAMBO data
        return [0]

    def _readFeedcode(self):
        return '1:GOOD,2:FLAGED'  # use fixed pseudo-code

    def _readBolCal(self):
        return 1.

    def _readUsrFrame(self, extname, subsnum ):
        # Adapt the IRAM system

        systemOf = self._readKeyword("SYSTEMOF",\
                                     extname=extname,\
                                     subsnum = subsnum)
        if systemOf == 'horizontalTrue':
            UsrFrame = "EQEQHO"
        elif systemOf == "projection":
            UsrFrame = "EQEQEQ"

        return UsrFrame


    def _readNUseFeedNIKA(self, key, subsnum, febe):
        return self._readKeywordFebe("NUSEFEED", extname="IMBF-frontend")[0]



import os
import numpy as np
from rea import ReaMBFits, ReaMBFitsReader, ReaDataEntity

path = '/data/abeelen/APEX/c-081.f-0015a-2008'
filename = 'APEX-20772-2008-06-08-C-081.F-0015A-2008'

path = '/data/abeelen/NIKA/Run10/TotalImbfits'
filename = 'iram30m-NIKA2mm-20141109s198-imb.fits'

subscans = []
readHe = 0
readAzEl0 = 0
readT = 0
readWind = 0
readBias = 0
readPWV = 0
channelFlag = 9
integrationFlag = 9

dataset = ReaMBFits.importDataset(os.path.join(path, filename))
reader = ReaMBFitsReader.createReader(dataset)
nSub = reader.openSubscan(subsnum=None)
febesDataset = reader.read("Febes")
useFebe = febesDataset[0]
basebandsDataset = reader.read("UseBand", febe=useFebe)
useBaseband = basebandsDataset[0]
subscansDataset = reader.read("Subscans")
useSubscans = subscansDataset
channelFlag = 1
timeline = ReaDataEntity.TimelineData()
# receiverArray = ReaDataEntity.ReceiverArray()
# scanParam = ReaDataEntity.ScanParameter()

try:
    timeline.ReceiverArray._ReceiverArray__fillFromMBFits(
        reader=reader,
        febe=useFebe,
     baseband=useBaseband,
     subscan=useSubscans[0],
     flag=channelFlag)
except:
    self = timeline.ReceiverArray
    febe = useFebe
    baseband = useBaseband
    subscan = useSubscans[0]
    flag = channelFlag

    useBand = reader.read("UseBand", febe=febe)
    indexBaseband = -1
    for iBand in xrange(len(useBand)):
        if useBand[iBand] == baseband:
            indexBaseband = iBand

    nChannels = reader.read("FebeFeed", febe=febe)
    self.NChannels = nChannels
    usedChannels = reader.read("UseFeed", febe=febe)[indexBaseband]
    if len(usedChannels.shape) == 3:
        usedChannels = usedChannels[0, :, 0]
    nbUsedChan = reader.read("NUseFeeds", febe=febe)[indexBaseband]
    usedChannels = usedChannels[
        :nbUsedChan]  # TODO: Is it really necessary ?!?
    refChannel = reader.read("RefFeed", febe=febe)
    feedType = reader.read("FeedType", febe=febe)[indexBaseband]
    feedString = reader.read("FeedCode", febe=febe)
    offX = reader.read("FeedOffX", febe=febe)
    offY = reader.read("FeedOffY", febe=febe)
    self.Gain = reader.read("FlatField", febe=febe)[indexBaseband]
    self.DCOff = reader.read("DCoffset", febe=febe)[indexBaseband]
    gains = reader._readGains(febe=febe)
    self.JyPerCount = reader.read("BolCalFc", febe=febe)
    self.UsedChannels = usedChannels
    self.CurrChanList = usedChannels
    self.NUsedChannels = len(usedChannels)
    self.RefChannel = refChannel
    self.Telescope.set(reader.read("Telescope", febe=febe),
                       float(reader.read("Diameter", febe=febe)),
                       float(reader.read("SiteLong", febe=febe)),
                       float(reader.read("SiteLat", febe=febe)),
                       float(reader.read("SiteElev", febe=febe)))

    # Still missing for NIKA
    freq = reader.read(
        "RestFreq",
        febe=febe,
     baseband=baseband,
     subsnum=subscan)
    self.DewCabin = reader.read("DewCabin", febe=febe)
    self.DewUser = reader.read("DewUser", febe=febe)
    self.DewExtra = reader.read("DewExtra", febe=febe, subsnum=subscan)


timeline.ScanParam.Telescope = timeline.ReceiverArray.Telescope

try:
    timeline.ScanParam._ScanParameter__fillFromMBFits(reader=reader,
                                                      febe=useFebe,
                                                      baseband=useBaseband,
                                                      subscans=useSubscans,
                                                      flag=integrationFlag,
                                                      readHe=readHe, readAzEl0=readAzEl0,
                                                      readT=readT, readWind=readWind,
                                                      readBias=readBias, readPWV=readPWV)
except:

    self = timeline.ScanParam
    reader = reader
    febe = useFebe
    baseband = useBaseband
    subscans = useSubscans
    flag = integrationFlag
    readHe = readHe
    readAzEl0 = readAzEl0
    readT = readT
    readWind = readWind
    readBias = readBias
    readPWV = readPWV
    self.ScanNum = reader.read("ScanNum")
    self.DateObs = reader.read("DateObs")
    self.ScanType = reader.read("ScanType")
    self.ScanMode = reader.read("ScanMode", subsnum=subscans[0])

    self.Object = reader.read("Object")
    self.Equinox = reader.read("Equinox")
    self.Basis = reader.read("Basis")
    self.Coord = reader.read("Coord")

    self.Frames = reader.read("UsrFrame", subsnum=subscans[0])

    # Missing for NIKA
    WobUsed = reader.read("WobUsed")

    self.DeltaCA = float(reader.read("DeltaCA"))
    self.DeltaIE = float(reader.read("DeltaIE"))

    self.TAIUTC = float(reader.read("TAIUTC"))
    self.UTCUT1 = float(reader.read("UTCUT1"))
    nIntegSubscan = {}
    nIntegTotal = 0
    for subscan in subscans:
        subscanWasOpened = reader.openSubscan(subsnum=subscan)
        nIntegSubscan[subscan] = reader.read("NInteg",
                                             subsnum=subscan,
                                             febe=febe,
                                             baseband=baseband)
        nIntegTotal += nIntegSubscan[subscan]
        if subscanWasOpened:
            reader.closeSubscan(subsnum=subscan)

    self.NInt = nIntegTotal
    self.NObs = reader.read("NObs")
    subscan = subscans[0]
    nbData = nIntegSubscan[subscan]
    subscanWasOpened = reader.openSubscan(subsnum=subscan)
    MJD = reader.read("MJD",
                      subsnum=subscan, febe=febe).astype(np.float64)
    LST = reader.read("LST",
                      subsnum=subscan,
                      febe=febe).astype(np.float32)
    Az = reader.read("Az",
                     subsnum=subscan,
                     febe=febe).astype(np.float32)
    El = reader.read("El",
                     subsnum=subscan,
                     febe=febe).astype(np.float32)
    # MeanRa = reader.read("Ra",
    #                      subsnum=subscan,
    #                      febe=febe).astype(np.float32)
    # MeanDec = reader.read("Dec",
    #                        subsnum=subscan,
    #                        febe=febe).astype(np.float32)
    LonOff = reader.read("LonOff",
                         subsnum=subscan,
                         febe=febe).astype(np.float32)
    LatOff = reader.read("LatOff",
                         subsnum=subscan,
                         febe=febe).astype(np.float32)
    BasLon = reader.read("BasLon",
                         subsnum=subscan,
                         febe=febe).astype(np.float32)
    BasLat = reader.read("BasLat",
                         subsnum=subscan,
                         febe=febe).astype(np.float32)
    Rot = reader.read("RotAngle",
                      subsnum=subscan,
                      febe=febe).astype(np.float32)
    # Missing for NIKA
    reFrac = reader.read("Refract", subsnum=subscan)
    obsType = reader.read("ObsType", subsnum=subscan, febe=febe)
    # Missing for NIKA
    SubscanDir = reader.read("ScanDir", subsnum=subscan, febe=febe)
    date_tmp = reader.read("SubsStart", subsnum=subscan, febe=febe)
    if subscanWasOpened:
        reader.closeSubscan(subsnum=subscan)

try:
    timeline._TimelineData__fillFromMBFits(reader=reader,
                                           febe=useFebe,
                                           baseband=useBaseband,
                                           subscans=useSubscans)
except:
    self = timeline
    reader = reader
    febe = useFebe
    baseband = useBaseband
    subscans = useSubscans

    nUseFeed = reader.read("NUseFeed",
                           subsnum=subscans[0],
                           febe=febe,
                           baseband=baseband)
    subscan = subscans[0]
    subsIndex = 0

    subscanWasOpened = reader.openSubscan(subsnum=subscan)
    subscanStart = self.ScanParam.SubscanIndex[0, subsIndex]
    subscanStop = self.ScanParam.SubscanIndex[1, subsIndex]
    tmpData = reader.read("Data",
                          subsnum=subscan,
                          febe=febe,
                          baseband=baseband)
    if subscanWasOpened:
        reader.closeSubscan(subsnum=subscan)



# Copyright (C) 2014
# Institut d'Astrophysique Spatiale
#
# Forked from the BoA project
#
# Copyright (C) 2002-2006
# Max-Planck-Institut fuer Radioastronomie Bonn
# Argelander Institut fuer Astronomie
# Astronomisches Institut der Ruhr-Universitaet Bochum
#
# Produced for the LABOCA project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#


class ReaError(Exception):

    """ A class used to generate exceptions related to Rea modules """

    def __init__(self, msg, value=None):
        self.value = value
        self.msg = msg

    def __str__(self):
        return repr(self.msg)



# Copyright (C) 2014
# Institut d'Astrophysique Spatiale
#
# Forked from the BoA project
#
# Copyright (C) 2002-2006
# Max-Planck-Institut fuer Radioastronomie Bonn
# Argelander Institut fuer Astronomie
# Astronomisches Institut der Ruhr-Universitaet Bochum
#
# Produced for the LABOCA project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#

"""
NAM: ReaConfig (module)
DES: Set the path/filename for i/o operation
"""

import os

inDir = '../fits'                 # input directory
outDir = './'                      # output directory

projID = 'None'        # Your ESO project Id
# Date of obs - in file name since 2007/8/13 (optional)
date = '2007-08-14'

inFile = ''                       # input file
outFile = ''                       # output file
rcpPath = os.getenv('BOA_HOME_RCP')  # Path for the rcp file when needed
if rcpPath is None:
    rcpPath = ""

online = 0               # Is boa used online or not
maxMessHandWeight = 4

goodFeedList = ['AC', 'DC', 'GOOD']
DEBUG = 2



#
# diagno.py
#
# Rea utilities with GUI
#
from .Bogli.Interface import *
from .Bogli import Plot

# external packages
from ppgplot import *
import string
from numpy import array
import copy


def openGui(data):
    """ Open a new window to display graphic interface,
    and start waiting loop. Various diagnostic plots
    are displayed when the user clicks on buttons.

    INP data: object of class ReaDataAnalyser.DataAna on
              which processing is performed """

    global chanList, chanRef, integRms

    devId = pgopen('/XW')
    pgask(0)   # no prompt when closing window
    pgpap(10, 0.3)             # window size (X in inch, ratio)
    pgscr(0, 0.85, 0.9, 0.85)  # background color
    pgsvp(0, 1, 0, 1)           # defines world: 100% window
    pgenv(0, 1000, 300, 0, 0, -2)  # world coordinate limits
    # window title
    pgsch(5)  # font size
    pgscf(2)  # font
    pgsci(4)  # font color
    pgptxt(500, 0, 0, 0.5, "BoA Diagnostic Tools")

    pgsci(16)  # black
    # defines 7 buttons: times, tel. pattern, signal, FFT, rms, diff, correl
    center = []
    sizes = []
    x0 = 80
    dx = 140
    y0 = 80
    for i in range(7):
        center.append((x0 + i * dx, y0))
        sizes.append((110, 80))
    label = ["Times|MJD,LST", "Telesc.|pattern", "Signal", "FFT",
             "RMS", "Diff.", "Correl."]

    # Add two big boutons for saving / exit
    center.append((125, 220))
    sizes.append((200, 80))
    label.append('Save last')
    center.append((350, 220))
    sizes.append((200, 80))
    label.append('Close')

    # add indications
    # button for selection of channels
    xx = array([300, 300, 530, 540, 550, 980, 980])
    yy = array([130, 145, 145, 205, 145, 145, 130])
    pgsci(4)
    pgline(xx, yy)
    center.append((540, 235))
    sizes.append((130, 50))
    label.append('channels')
    # button for reference channel
    xx = array([720, 720, 840, 850, 860, 980, 980])
    yy = array([155, 170, 170, 205, 170, 170, 155])
    pgline(xx, yy)
    center.append((850, 235))
    sizes.append((130, 50))
    label.append('Ref chan')
    # RMS: number of integrations
    pgline(array([670, 670]), array([125, 160]))
    center.append((660, 180))
    sizes.append((80, 50))
    label.append('integ.')

    # default values
    pgsfs(1)
    pgsci(16)
    pgrect(475, 605, 270, 300)
    pgrect(785, 915, 270, 300)
    pgrect(620, 700, 215, 245)
    pgsci(2)
    pgscf(1)
    pgsch(2)
    chanList = 'all'
    chanRef = data.ReceiverArray.RefChannel
    integRms = 10
    pgptxt(485, 290, 0, 0, str(chanList))
    pgptxt(795, 290, 0, 0, str(chanRef))
    pgptxt(630, 235, 0, 0, str(integRms))

    # display buttons and wait for clicks
    boutons = Fenetre(1, center, sizes, label, (0, 10),
                      font=3., coltxt=1, colfond=14, family=2)
    boutons.dessine(new=0)

    again = 1
    while(again):
        numChoix = boutons.saisie()
        if numChoix == 8:
            again = 0
        else:
            processPlot(numChoix, data, devId)

    pgclos()


def processPlot(num, data, prevDev):
    """
    DES: do one of the diagnostic plots
    INP: (int) num = type of diagnostic
         (DataAna object) data = the data
         (int) prevdev = device number of main window
    """
    if num == 0:
        plotTimes(data)
    elif num == 1:
        plotPattern(data)
    elif num == 2:
        plotSignal(data)
    elif num == 4:
        plotRMS(data)
    elif num == 5:
        plotDiff(data)
    elif num == 6:
        plotCorrel(data)

    elif num == 9:
        # select channels
        selectChan(data)
    elif num == 10:
        # select ref. channel
        selectRef(data)
    elif num == 11:
        # set nb. of integrations to compute rms
        setInteg(data)

    else:
        print num, 'not implemented'

    pgslct(prevDev)

#
# individual plotting routines
#


def plotTimes(data):

    pgopen('/XW')
    pgask(0)

    lst = data.ScanParam.LST
    MJD_renorm = (data.ScanParam.MJD - data.ScanParam.MJD[0]) * 86400.

    Plot.plot(lst, style='l', labelX='Integration number',
              labelY='LST [sec]',
              caption='Scan ' + str(data.ScanParam.ScanNum) +
              ' - ' + data.ScanParam.Object +
              ' - ' + data.ReceiverArray.FeBe +
              ' - ' + data.ReceiverArray.Telescope.Name)
    print "Press return for next plot"
    raw_input()

    Plot.plot(MJD_renorm, style='l', labelX='Integration number',
              labelY='MJD - MJD[0] [sec]',
              caption='Scan ' + str(data.ScanParam.ScanNum) +
              ' - ' + data.ScanParam.Object +
              ' - ' + data.ReceiverArray.FeBe +
              ' - ' + data.ReceiverArray.Telescope.Name)

    dt = []
    nbInteg = len(lst)
    for i in range(nbInteg - 1):
        dt.append(MJD_renorm[i + 1] - MJD_renorm[i])

    print "Press return for next plot"
    raw_input()
    Plot.plot(1. / array(dt), style='p', labelX='Integration number',
              labelY='Sampling rate [Hz]',
              caption='Scan ' + str(data.ScanParam.ScanNum) +
              ' - ' + data.ScanParam.Object +
              ' - ' + data.ReceiverArray.FeBe +
              ' - ' + data.ReceiverArray.Telescope.Name)

    print "Press return for next plot"
    raw_input()

    Plot.plot(lst, MJD_renorm, style='p', labelX='LST [sec]',
              labelY='MJD - MJD[0] [sec]',
              caption='Scan ' + str(data.ScanParam.ScanNum) +
              ' - ' + data.ScanParam.Object +
              ' - ' + data.ReceiverArray.FeBe +
              ' - ' + data.ReceiverArray.Telescope.Name)
    print "Press return to close window"
    raw_input()
    pgclos()


def plotPattern(data):

    pgopen('/XW')
    pgask(0)

    data.ScanParam.plotAzEl()
    data.ScanParam.plotAzEl(style='p', overplot=1, ci=2)

    print "Press return for next plot"
    raw_input()

    data.ScanParam.plotAzElOffset()
    data.ScanParam.plotAzElOffset(style='p', overplot=1, ci=2)
    print "Press return to close window"
    raw_input()
    pgclos()


def plotSignal(data):

    # simply use current list of channels
    pgopen('/XW')
    pgask(0)

    data.signal()
    print "Press return for next plot"
    raw_input()
    data.signal(plotMap=1)
    print "Press return to close window"
    raw_input()
    pgclos()


def plotFFT(data):
    # to be done...
    # data.fft([17],mjd=1,style='l')
    pass


def plotRMS(data):

    global integRms
    slideRms = data.slidingRms(nbInteg=integRms)
    pgopen('/XW')
    pgask(0)
    Plot.draw(slideRms, wedge=1, caption='Signal RMS', labelY='Channel #')
    print "Press return to close window"
    raw_input()
    pgclos()


def plotDiff(data):

    global chanRef

    backup = copy.copy(data.Data)
    # subtract signal of chanRef
    for n in data.ReceiverArray.CurrChanList:
        data.Data[:, n - 1] = data.Data[:, n - 1] - backup[:, chanRef - 1]

    pgopen('/XW')
    pgask(0)
    data.signal()
    print "Press return for next plot"
    raw_input()
    data.signal(plotMap=1)
    print "Press return to close window"
    raw_input()
    pgclos()

    # restore data to initial state
    data.Data = copy.copy(backup)


def plotCorrel(data):

    global chanRef

    pgopen('/XW')
    pgask(0)
    # simply use current list of channels
    data.plotCorrel(chanRef=chanRef)
    print "Press return to close window"
    raw_input()
    pgclos()


#
# routines to set paramaters
#
def selectChan(data):
    """ open a box to fill by the user with valid
    list of channels. Return list when valid

    INP: (object) data = data to process
    """

    global chanList

    answer = fenetreInteractive(x0=475, y0=270, x1=605, y1=300,
                                prevText=chanList, bgCol=16)
    chan = answer
    # interpret answer like '1-20'
    if '-' in chan:
        words = string.split(chan, '-')
        try:
            toList = range(int(words[0]), int(words[1]) + 1)
            chan = toList
        except ValueError:  # not integers
            data.MessHand.warning("Could not interpret input list")
    elif chan != 'all':  # one integer, or coma-separated list of int
        num = string.split(chan, ',')
        toList = []
        try:
            for n in num:
                toList.append(int(n))
            chan = toList
        except ValueError:  # not integers
            data.MessHand.warning("Could not interpret input list")

    # message handler will write error message if not
    # a valid list
    data.ReceiverArray.setCurrChanList(chan)
    chanList = answer


def selectRef(data):
    """ open a box to input ref. channel number
    INP: (object) data = data to process
    """

    global chanRef

    again = 1
    while(again):
        answer = fenetreInteractive(x0=785, y0=270, x1=915, y1=300,
                                    prevText=str(chanRef), bgCol=16)
        try:
            newRef = int(answer)
            if newRef in data.ReceiverArray.UsedChannels:
                chanRef = newRef
                again = 0
            else:
                data.MessHand.warning("Channel not in use")
        except ValueError:
            # not an integer
            data.MessHand.warning("Could not interpret input value")


def setInteg(data):
    """ open a box to input number of integrations for computing rms
    INP: (object) data = data to process
    """

    global integRms

    again = 1
    while(again):
        answer = fenetreInteractive(x0=620, y0=215, x1=700, y1=245,
                                    prevText=str(integRms), bgCol=16)
        try:
            newInteg = int(answer)
            if newInteg > 1:  # need at least 2 integ.
                integRms = newInteg
                again = 0
            else:
                # should use message handler
                data.MessHand.warning(
                    "Need at least two integrations to compute rms")
        except ValueError:
            # not an integer
            data.MessHand.warning("Could not interpret input value")



import numpy as np
from .fortran import fStat
from .Bogli import DeviceHandler, Plot, MultiPlot


def telescopeSpeed(ScanParam, wait=1, style='p'):
    MJD = ScanParam.get('MJD').astype(np.float64)
    azimuth = ScanParam.get('Azimuth').astype(np.float64)
    elevation = ScanParam.get('Elevation').astype(np.float64)

    # Basic plots
    # Azimuth/Elevation
    azStat = fStat.basicstat(azimuth)
    limitsY = np.array([-1, 1]) * 3 * azStat[1] + azStat[0]
    Plot.plot(MJD, azimuth, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='azimuth (deg)',
              limitsY=limitsY,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    elStat = fStat.basicstat(elevation)
    limitsY = np.array([-1, 1]) * 3 * elStat[1] + elStat[0]
    Plot.plot(MJD, elevation, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='elevation (deg)',
              limitsY=limitsY,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    limitsX = np.array([-1, 1]) * 3 * azStat[1] + azStat[0]
    limitsY = np.array([-1, 1]) * 3 * elStat[1] + elStat[0]
    Plot.plot(azimuth, elevation, style=style,
              labelX='azimuth (deg)', labelY='elevation (deg)',
              limitsX=limitsX, limitsY=limitsY,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    # Speed

    dt = MJD[1:] - MJD[:-1]
    azimuthSpeed = (azimuth[1:] - azimuth[:-1]) / dt
    elevationSpeed = (elevation[1:] - elevation[:-1]) / dt

    azSpeedStat = fStat.basicstat(azimuthSpeed)
    elSpeedStat = fStat.basicstat(elevationSpeed)

    limitsY = np.array([-1, 1]) * 4 * azSpeedStat[1] + azSpeedStat[0]
    Plot.plot(MJD[:-1], azimuthSpeed, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='Azimuth Speed (deg/s)',
              limitsY=limitsY,
              caption=ScanParam.caption())
    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    limitsY = np.array([-1, 1]) * 4 * elSpeedStat[1] + elSpeedStat[0]
    Plot.plot(MJD[:-1], elevationSpeed, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='Elevation Speed (deg/s)',
              limitsY=limitsY,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    limitsX = np.array([-1, 1]) * 4 * azSpeedStat[1] + azSpeedStat[0]
    limitsY = np.array([-1, 1]) * 4 * elSpeedStat[1] + elSpeedStat[0]
    Plot.plot(azimuthSpeed, elevationSpeed, style=style,
              labelX='Azimuth Speed (deg/s)', labelY='Elevation Speed (deg/s)',
              limitsX=limitsX, limitsY=limitsY, aspect=1,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    # Accelation
    azimuthAccel = (azimuthSpeed[1:] - azimuthSpeed[:-1]) / dt[:-1]
    elevationAccel = (elevationSpeed[1:] - elevationSpeed[:-1]) / dt[:-1]

    azAccelStat = fStat.basicstat(azimuthAccel)
    elAccelStat = fStat.basicstat(elevationAccel)

    limitsY = np.array([-1, 1]) * 5 * azAccelStat[1] + azAccelStat[0]
    Plot.plot(MJD[:-1], azimuthAccel, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='Azimuth Accel (deg/s2)',
              limitsY=limitsY,
              caption=ScanParam.caption())
    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    limitsY = np.array([-1, 1]) * 5 * elAccelStat[1] + elAccelStat[0]
    Plot.plot(MJD[:-1], elevationAccel, style=style,
              labelX='MJD-MJD[0] (sec)', labelY='Elevation Accel (deg/s2)',
              limitsY=limitsY,
              caption=ScanParam.caption())

    if wait:
        print "Press Enter to proceed"
        raw_input()
    Plot.clear()

    limitsX = np.array([-1, 1]) * 5 * azAccelStat[1] + azAccelStat[0]
    limitsY = np.array([-1, 1]) * 5 * elAccelStat[1] + elAccelStat[0]
    Plot.plot(azimuthAccel, elevationAccel, style=style,
              labelX='Azimuth Accel (deg/s2)', labelY='Elevation Accel (deg/s2)',
              limitsX=limitsX, limitsY=limitsY, aspect=1,
              caption=ScanParam.caption())
